#===========================================================================
#  FileName: VerifyEverything.ksh
#    Author: Craig Schneider
# Orig Date: 06/14/00
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Run all of the verification scripts and executables.
#===========================================================================

START_TIME=`date +"%F %T"`
export TESTCASE_NUMBER=0
export TESTCASE_FORMAT=normal
export CODE_COVERAGE_EXE=
export CODE_COVERAGE_CMD=
export VERIFY_EVERYTHING_RUNNING=TRUE
CODE_COVERAGE=FALSE
RUN_TESTS_ARGS=
VERIFY_ORDER_DEFAULT="VerifyTestOrder"
VERIFY_ORDER_OVERRIDE=
VERIFY_MULTI_DIR_MODE=FALSE
VERIFY_DIRECTORY_LIST=
EZ_DRYRUN=FALSE

#==========================================================================
# If the user aborts at any time, which is very likely, cleanup any
# temporary files.
#==========================================================================
function CleanUp
{
   rm -f $VERIFICATION_LOG
}
trap "CleanUp ; exit 1" 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15

#==========================================================================
# Display the usage message to the user.
#==========================================================================
function Usage
{
   cat <<!
Usage: $0 [-psv] [-yes | -no] [-l logfile] [-f filename] [-dryrun] [-quiet]
          [-d directory] [-no_hosts] [-CodeCoverage] [-test_name TestName ...]

-CodeCoverage: Specifies that code coverage results are produced.
           -d: Specify a directory to search for verify files or verify sub
               directories in.  Defaults to the current directory.  Note that
               multiple directories can be specified by using multiple 
               '-d directory' options.
      -dryrun: Display the commands to be executed, without executing them.
           -f: Specify the filename containing the list of verify scripts
               to execute in the order listed within the file.  If a file
               of the name '$VERIFY_ORDER_DEFAULT' exists in the current
               directory, it is used automatically.
           -l: Specify the name of a file to log the verification results to.
    -no_hosts: Do not output information regarding the host machine used.
          -no: For any user query, assume the user answered no.
         -psv: Output verification results in pipe separated value format.
       -quiet: Do not output anything if nothing was found to verify.
         -yes: For any user query, assume the user answered yes.
   -test_name: Specifies a list of test names to be run.
!
   exit 1
}

typeset -l LOWERCASE_ARG
while [ "$#" -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
       -quiet) export QUIET_MODE=TRUE
               shift ;;
-codecoverage) RUN_TESTS_ARGS="$RUN_TESTS_ARGS -CodeCoverage"
               CODE_COVERAGE_EXE=CodeCoverage.ksh
               CODE_COVERAGE_CMD=CodeCoverage.ksh
               CODE_COVERAGE=TRUE
               shift ;;
      -dryrun) export EZ_DRYRUN=TRUE
               shift ;;
         -psv) export TESTCASE_FORMAT=psv
               echo "Number|Name|Details|Status"
               shift ;;
         -yes) export TESTCASE_ANSWER=y
               shift ;;
          -no) export TESTCASE_ANSWER=n
               shift ;;
    -no_hosts) export NO_HOST_INFO=TRUE
               shift ;;
           -d) VERIFY_DIRECTORY_LIST="$VERIFY_DIRECTORY_LIST $2"
               shift
               shift ;;
           -f) VERIFY_ORDER_OVERRIDE=$2
               shift
               shift ;;
           -l) VERIFY_LOG_FILE=$2
               shift
               shift ;;
   -test_name) RUN_TESTS_ARGS="$RUN_TESTS_ARGS -t $2"
               shift
               shift ;;
            *) Usage
               shift ;;
   esac
done

if [ "$VERIFY_DIRECTORY_LIST" = "" ] ; then
   VERIFY_DIRECTORY_LIST=.
fi

. ${EZ_VERIFY_ROOT-.}/VerifyFunctions.ksh $0

#==========================================================================
# Determine which VerifyOrderFile to use if any.
#==========================================================================
function SetupVerifyOrderFile
{
   VERIFY_ORDER_FILE=
   if [ "$VERIFY_ORDER_OVERRIDE" != "" ] ; then
      if [ -f "$VERIFY_ORDER_OVERRIDE" ] ; then
         VERIFY_ORDER_FILE=$VERIFY_ORDER_OVERRIDE
      else
         if [ "$1" = TestOnly ] ; then
            return 1
         fi
         if [ "$VERIFY_MULTI_DIR_MODE" = TRUE ] ; then
            echo -n "Warning: Verify order file $VERIFY_ORDER_OVERRIDE "
            echo "not found in dir $PWD!"
         else
            echo -n "ERROR: Verify order file $VERIFY_ORDER_OVERRIDE "
            echo "not found in dir $PWD!"
            exit 1
         fi
      fi
   fi

   if [ "$VERIFY_ORDER_FILE" = "" -a  -f "$VERIFY_ORDER_DEFAULT" ] ; then
      VERIFY_ORDER_FILE=$VERIFY_ORDER_DEFAULT
   fi

   if [ "$EZ_DRYRUN" = TRUE ] ; then
      if [ "$VERIFY_ORDER_FILE" != "" ] ; then
         echo "Debug: Using verify order file $VERIFY_ORDER_FILE."
      else
         echo "Debug: No verify order file found. "
      fi
   fi
}

#==========================================================================
# Determine the list of verify files to run and in which order.
#==========================================================================
function SetupVerifyFileList
{
   SetupVerifyOrderFile $1

   if [ "$VERIFY_ORDER_FILE" != "" ] ; then
      FILE_LIST=`cat $VERIFY_ORDER_FILE`
   else
      if [ -d bin -a "$EZ_OS_TYPE" = windows ] ; then
         FILE_LIST=`echo bin/v_*`
         if [ "$FILE_LIST" = "bin/v_*" ] ; then
            FILE_LIST=`ls bin/*Test.dll | egrep -v C4UnitTest.dll`
         fi
      else
         FILE_LIST=`echo v_*`
      fi
   fi
   VERIFY_FILE_LIST=
   for SINGLE_FILE in $FILE_LIST
   do
      # if this is coming back false, might need to dos2unix the list file.
      if [ -f "$SINGLE_FILE" ] ; then
         IS_VERIFY_FILE=FALSE
         EXTENSION=${SINGLE_FILE##*.}
         TEST_DLL_STRIPPED="${SINGLE_FILE%Test.dll}"
         if [ "$EXTENSION" = dll -a \
              "$TEST_DLL_STRIPPED" != "$SINGLE_FILE" ] ; then
            IS_VERIFY_FILE=TRUE
         fi
         if [ "$EXTENSION" = ksh -o "$EXTENSION" = sh ] ; then
            IS_VERIFY_FILE=TRUE
         fi
         if [ "$EXTENSION" = "$SINGLE_FILE" -a "$EZ_OS_TYPE" != windows ] ; then
            IS_VERIFY_FILE=TRUE
         fi
         if [ "$EXTENSION" = exe -a "$EZ_OS_TYPE" = windows ] ; then
            IS_VERIFY_FILE=TRUE
            if [ "${SINGLE_FILE%.vshost.exe}" != "$SINGLE_FILE" ] ; then
               IS_VERIFY_FILE=FALSE
            fi
         fi
         if [ "$EXTENSION" = cs -a "$EZ_OS_TYPE" = windows ] ; then
            IS_VERIFY_FILE=TRUE
         fi
         # skip files ending in ~ since they are emacs backup files.
         if [ "${SINGLE_FILE%\~}" != "$SINGLE_FILE" ] ; then
            IS_VERIFY_FILE=FALSE
         fi
         if [ "$IS_VERIFY_FILE" = TRUE ] ; then
            VERIFY_FILE_LIST="$VERIFY_FILE_LIST $SINGLE_FILE"
         fi
      fi
   done
   if [ "$VERIFY_FILE_LIST" = "" ] ; then
      if [ "$1" = TestOnly ] ; then
         return 1
      fi
      if [ "$VERIFY_MULTI_DIR_MODE" = TRUE ] ; then
         echo "Warning: No verify files found in directory $PWD!"
         return 1
      else
         echo "ERROR: No verify files found in directory $PWD!"
         exit 1
      fi
   fi
   return 0
}

#==========================================================================
# Execute all of the verify files in the current directory.
#==========================================================================
function VerifySingleDirectory
{
   SetupVerifyFileList
   if [ $? -ne 0 ] ; then
      return
   fi

   if [ -f VerifyInitialize.ksh ] ; then
      VerifyInitialize.ksh
      if [ $? -ne 0 ] ; then
         echo "ERROR: VerifyInitialize.ksh failed."
         exit 1
      fi
   fi

   for VERIFY_FILE_NAME in $VERIFY_FILE_LIST
   do
      EXTENSION=${VERIFY_FILE_NAME##*.}
      VERIFY_CMD="$VERIFY_FILE_NAME"
      CODE_COVERAGE_CMD=
      if [ "$CODE_COVERAGE_EXE" != "" ] ; then
         MASTER_FILE_NAME="${VERIFY_FILE_NAME%.ksh}.coverage"
         CODE_COVERAGE_CMD="$CODE_COVERAGE_EXE -MasterFileName $MASTER_FILE_NAME"
      fi 
      if [ "$EXTENSION" = dll ] ; then
         VERIFY_CMD="RunDllTests.sh -NoSummary $VERIFY_CMD $RUN_TESTS_ARGS"
      fi
      if [ "$EZ_DRYRUN" = TRUE ] ; then
         echo "Would execute $VERIFY_CMD"
      else
         $VERIFY_CMD | tee -a $VERIFICATION_LOG
      fi
   done

   if [ -f VerifyFinalize.ksh ] ; then
      VerifyFinalize.ksh
      if [ $? -ne 0 ] ; then
         echo "ERROR: VerifyFinalize.ksh failed."
         exit 1
      fi
   fi
}

#==========================================================================
# Main code, determine if we are in single directory or multi directory
# mode and run the appropriate verifies.
#==========================================================================
export VERIFICATION_LOG=$PWD/MultiVerify.log
rm -f $VERIFICATION_LOG
touch $VERIFICATION_LOG

SetupVerifyFileList "TestOnly"
if [ $? -ne 0 ] ; then        # no v_* files in current dir do recursive mode
   VERIFY_MULTI_DIR_MODE=TRUE
   ORIG_SAVE_DIR=$PWD
   for VERIFY_DIRECTORY in $VERIFY_DIRECTORY_LIST
   do
      cd $VERIFY_DIRECTORY
      # first check for directories
      VERIFY_SUB_DIR_LIST=`find . -follow \( -type d -o -type l \) -a \
                                   \( -name v_\* -o -name \*Test \) \
                                  -print | egrep -v 'C4UnitTest$|DNetTest$'`
      if [ "$VERIFY_SUB_DIR_LIST" = "" ] ; then
         # next check for files
         VERIFY_SUB_DIR_LIST=`find . -follow \( -type f \) -a \
                                   \( -name v_\* -o -name \*Test.dll \) \
                                   -print | egrep -v "C4UnitTest"`
         if [ "$VERIFY_SUB_DIR_LIST" = "" ] ; then
            if [ "$QUIET_MODE" != TRUE ] ; then
               echo -n "ERROR: No verify files or verify sub directories found "
               if [ "$VERIFY_DIRECTORY" = "." ] ; then
                  echo "in $PWD!"
               else
                  echo "in $VERIFY_DIRECTORY!"
               fi
            fi
            exit 1
         fi
         VERIFY_SUB_DIR_LIST=$VERIFY_DIRECTORY
      fi
      VERIFY_SAVE_DIR=$PWD
      for VERIFY_SUB_DIR in $VERIFY_SUB_DIR_LIST
      do
         cd $VERIFY_SUB_DIR
         if [ "${VERIFY_SUB_DIR##*/}" = bin ] ; then
            echo "Successfully changed to verify directory $VERIFY_SUB_DIR."
         else
            echo "Successfully changed to verify directory ${VERIFY_SUB_DIR##*/}."
         fi
         VerifySingleDirectory
         cd $VERIFY_SAVE_DIR
      done
      cd $ORIG_SAVE_DIR
   done
else
   VerifySingleDirectory
fi

#==========================================================================
# Output final summary statistics and execution information.
#==========================================================================
typeset -R4 TOTAL_PASSED TOTAL_FAILED
TOTAL_PASSED=`grep "$VERIFY_PASSED" $VERIFICATION_LOG | wc -l | awk '{print $1}'`
TOTAL_FAILED=`egrep "$VERIFY_FAILED|FAILED: " $VERIFICATION_LOG | wc -l | awk '{print $1}'`
let TOTAL_TEST_CASES=TOTAL_PASSED+TOTAL_FAILED

if [ "$VERIFY_LOG_FILE" != "" ] ; then
   cp $VERIFICATION_LOG $VERIFY_LOG_FILE
fi
CleanUp

if [ "$EZ_ACTIVE_RELEASE" != "" ] ; then
   RELEASE_INFO=" under release $EZ_ACTIVE_RELEASE"
fi
if [ "$IVVIEW" != "" ] ; then
   RELEASE_INFO=" under the $IVVIEW"
fi

if [ "$NO_HOST_INFO" != TRUE ] ; then
   HOST_INFO=" on $(uname -n)"
fi

if [ "$CODE_COVERAGE" = TRUE -a "$EZ_OS_TYPE" != linux ] ; then
   IGNORED_ASSEMBLIES_FILE="$EZ_CODE_COVERAGE_ROOT/${EZ_ACTIVE_RELEASE%_*}.IgnoredAssemblies"
   if [ -f "$IGNORED_ASSEMBLIES_FILE" ] ; then
      IGNORED_ASSEMBLY_NAMES=`cat "$IGNORED_ASSEMBLIES_FILE"`
      IGNORED_ASSEMBLY_ARGS="-IgnoredAssemblyNames $IGNORED_ASSEMBLY_NAMES"
   fi
   END_TIME=`date +"%F %T"`
   C4.sh -Coverage Stats -f "$EZ_CODE_COVERAGE_ROOT/*.coverage" \
         -PerfStatsDirectory S:/Archive/PerfStatsCodeCoverage \
         -StartTime "$START_TIME" -EndTime "$END_TIME" \
         -SourceProgramType $EZ_ACTIVE_RELEASE $IGNORED_ASSEMBLY_ARGS \
         -AllowNoFiles -Verbose
   
   C4.sh -Coverage Merge -f "$EZ_CODE_COVERAGE_ROOT/*.coverage" \
                         -o "$EZ_CODE_COVERAGE_ROOT/Merged.coverage.xml" \
                         -AllowNoFiles -Verbose
fi

if [ "$QUIET_MODE" != TRUE ] ; then
   cat << !
*****************************************************************************
!
fi

cat <<!
Successfully executed test cases as $LOGNAME on $(date "+%b %e, %Y").
Successfully executed test cases${RELEASE_INFO}${HOST_INFO}.
Successfully Verified   $TOTAL_PASSED of the $TOTAL_TEST_CASES test cases.
!

if [ $TOTAL_FAILED -ne 0 ] ; then
echo "Encountered Failures on $TOTAL_FAILED of the $TOTAL_TEST_CASES test cases."
fi

if [ "$QUIET_MODE" != TRUE ] ; then
   cat << !
*****************************************************************************
!
fi

   exit 1
fi

exit 0
