#===========================================================================
# FileName: <<FileName>>
#   Author: Craig Schneider
#  Created: 2004-11-16
# Synopsis: Recursively compare 2 directories with options to syncronize.
# <<Copyright>>
#===========================================================================

. $EZ_SHARED_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: $0 [-c CreationCommand] [-certify FileName] [-d DestructionCommand]
          [-diff] [-diffs] [-dir_tokens] [-dll_diffs] [-dos2unix]
          [-dos2unixcmp]
          [-exclude (dirs | exts | files | rdirs) (ExclusionList | -f FileName)]
          [-exclude_exe] [-excluded] [-f FileListFile] [-filever] 
          [-force | -no | -no_prompt | -yes ] [-file_fixer Fixer] [-fix_cmd]
          [-forward_only] [-groups] [-i] [-ii] [-identical] [-name_fixer Fixer]
          [-no_excluded] [-no_destroy] [-no_different] [-no_missing]
          [-no_recompile] [-owners] [-perms] [-stats | -no_stats]
          [-tmp WorkDirectory] [-dryrun] [-quiet | -verbose] [-sync]
          SourceDir TargetDir

          -c: Creation command used when stabalization creates a new TargetFile.
                 chmod:  After creating the TargetFile, perform a chmod 777.
                 deduce: Deduce if TargetFile should be source controled.
                 none:   Do nothing after creating the TargetFile (default).
                 put:    After creating the TargetFile, perform a src put -c.
    -certify: File containing difference certification information.
          -d: Destruction command used when stabalization destroys a TargetFile.
                 deduce:      Deduce if TargetFile should be src destroyed.
                 rm:          Use rm to destroy the TargetFile (default).
                 src_delete:  Use src delete to destroy the TargetFile.
                 src_destroy: Use src destroy to destroy the TargetFile.
        -diff: Use diff instead of cmp to perform the initial comparision.
       -diffs: Display differences of files before the user is prompted.
  -dir_tokens: Display the tokenized path (shorter) to files and dirs.
   -dll_diffs: Display differences of .dll files before the user is prompted.
    -dos2unix: dos2unix versions of files are diffed and stabalized.
 -dos2unixcmp: dos2unix versions of files are diffed, originals are stabalized.
      -dryrun: Display the commands to be executed, without executing them.
     -exclude: Exclude a list of files/dirs from comparision.  The exclusion
               list is specified as either a pipe separated list or the
               name of a file containing one entry per line.
                 dirs:  Directory names to be excluded but not recursively.
                 exts:  File name extensions to be excluded anywhere.
                 files: File names to be excluded anywhere.
                 rdirs: Directory names to be recursively excluded (slower).
 -exclude_exe: Exclude executable files from comparision.
    -excluded: Display the names of files that were excluded.
           -f: Name of the file that contains the list of files to compare.
     -filever: Allow the use of the filever when displaying file differences.
  -file_fixer: Function to invoke to perform advanced file stabalizing.
     -fix_cmd: Display the src stabalize command for files not updated.
       -force: All user questions will be implicitly answered with a yes.
-forward_only: Only sync from the source to the target.
      -groups: Ensure that the files group owners are identical.
           -i: Ignore the case of the file/directory names being compared.
          -ii: Ignore the case of the file content being compared.
   -identical: Display the names of files that were identical.
  -name_fixer: The function to call to fix the file name.
          -no: All user questions will be implicitly answered with a no.
 -no_excluded: Do not display the names of files that were excluded.
  -no_destroy: Do not destroy TargetFile if SourceFile does not exist.
-no_different: Do not display the names of files or dirs that were different.
  -no_missing: Do not display the names of files or dirs that were missing.
   -no_prompt: All user questions will be implicitly answered with a no.
-no_recompile: Do not display the names of files that were recompiled.
    -no_stats: Turn off statistics about the comparisions made.
      -owners: Ensure that the files owners are identical.
       -perms: Ensure that file permissions are identical.
        -sync: Prompt for changes needed to syncronize the source and target.
    SourceDir: The source directory to be used for comparisions.
    TargetDir: The target directory to be used for comparisions.
         -tmp: Directory used for any temporary files created during execution.
       -quiet: Suppress non critical messages about operations being performed.
     -verbose: Display verbose messages about operations being performed.
         -yes: All user questions will be implicitly answered with a yes.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi
if [ $# -eq 0 ] ; then
   Usage
fi

EZ_DRYRUN=FALSE
EZ_VERBOSITY=

CERTIFY_FILE_NAME=
DIFF_DIR_AUTO_ANSWER=
DISPLAY_DIFFERENT=TRUE
DISPLAY_DIFFS=FALSE
DISPLAY_DIR_TOKENS=FALSE
DISPLAY_EXCLUDED=FALSE
DISPLAY_FIX_COMMAND=FALSE
DISPLAY_IDENTICAL=FALSE
DISPLAY_MISSING=TRUE
DISPLAY_RECOMPILED=TRUE
DISPLAY_STATISTICS=FALSE
DO_OWN_COMPARES=TRUE
EXCLUDE_EXE=FALSE
EXCLUDE_FILES_FILE_NAME=
EXCLUDE_DIRS_LIST=
EXCLUDE_DIRS_FILE_NAME=
EXCLUDE_EXTS_LIST=
EXCLUDE_RDIRS_LIST=
EXCLUDE_RDIRS_FILE_NAME=
EXCL_OPT=
EXCL_DIRS_OPT=
EXCL_RDIRS_OPT=
FILE_LIST_FILE_NAME=
FILE_NAMES_ONLY=FALSE
FORWARD_ONLY=FALSE
IGNORE_CASE=FALSE
NAME_FIXER=
SOURCE_DIRECTORY_NAME=
STABALIZER="src.sh stabalize"
STABALIZE_OPTS="-c deduce -d deduce"
SYNCRONIZATION_MODE=READONLY
TARGET_DIRECTORY_NAME=
TMP_DIRECTORY="$TMP"
USE_DIFF_EXE=FALSE

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -c) STABALIZE_OPTS="$STABALIZE_OPTS -c $2"
          shift
          shift ;;
      -certify)
          CERTIFY_FILE_NAME="$2"
          STABALIZE_OPTS="$STABALIZE_OPTS -certify $2"
          shift
          shift ;;
      -d) STABALIZE_OPTS="$STABALIZE_OPTS -d $2"
          shift
          shift ;;
      -diff) STABALIZE_OPTS="$STABALIZE_OPTS -diff"
            USE_DIFF_EXE=TRUE
            shift ;;
      -diffs) STABALIZE_OPTS="$STABALIZE_OPTS -diffs"
            DISPLAY_DIFFS=TRUE
            shift ;;
      -dir_tokens) 
            DISPLAY_DIR_TOKENS=TRUE
            shift ;;
      -dll_diffs) STABALIZE_OPTS="$STABALIZE_OPTS -dll_diffs"
            shift ;;
      -dos2unix) STABALIZE_OPTS="$STABALIZE_OPTS -dos2unix"
            shift ;;
      -dos2unixcmp) STABALIZE_OPTS="$STABALIZE_OPTS -dos2unixcmp"
            shift ;;
      -dryrun)
            EZ_DRYRUN=TRUE
            shift ;;
      -exclude)
            if [ "$3" = "-f" ] ; then
               if [ "$2" = files ] ; then
                  EXCLUDE_FILES_FILE_NAME="$4"
                  STABALIZE_OPTS="$STABALIZE_OPTS -e $EXCLUDE_FILES_FILE_NAME"
               elif [ "$2" = dirs ] ; then
                  EXCL_DIRS_OPT=",x"
                  EXCLUDE_DIRS_FILE_NAME="$4"
                  if [ ! -f "$EXCLUDE_DIRS_FILE_NAME" ] ; then
                     touch "$EXCLUDE_DIRS_FILE_NAME"
                  fi
                  EXCLUDE_DIRS_LIST="|$(cat $EXCLUDE_DIRS_FILE_NAME | \
                                     tr '\n' '|')"
               elif [ "$2" = rdirs ] ; then
                  EXCL_RDIRS_OPT=",X"
                  EXCLUDE_RDIRS_FILE_NAME="$4"
                  if [ ! -f "$EXCLUDE_RDIRS_FILE_NAME" ] ; then
                     touch "$EXCLUDE_RDIRS_FILE_NAME"
                  fi
                  EXCLUDE_RDIRS_LIST=`cat $EXCLUDE_RDIRS_FILE_NAME`
               else
                  Usage "Invalid -excluded option $2 when used with -f."
               fi
               shift
               shift
               shift
            else
               if [ "$2" = dirs ] ; then
                  EXCLUDE_DIRS_LIST="|$3|"
               elif [ "$2" = exts ] ; then
                  EXCLUDE_EXTS_LIST="|$3|"
               elif [ "$2" = files ] ; then
                  EXCLUDE_FILES_FILE_NAME="$TMP_DIRECTORY/excl.$$"
                  echo "$3|" | tr '|' '\n' > $EXCLUDE_FILES_FILE_NAME
                  STABALIZE_OPTS="$STABALIZE_OPTS -ee $EXCLUDE_FILES_FILE_NAME"
               elif [ "$2" = rdirs ] ; then
                  EXCLUDE_RDIRS_LIST=`echo "$3|" | tr '|' '\n'`
               else
                  Usage "Invalid -excluded option $2."
               fi
               shift
               shift
            fi
            shift ;;
      -exclude_exe)
            EXCLUDE_EXE=TRUE
            shift ;;
      -excluded)
            DISPLAY_EXCLUDED=TRUE
            shift ;;
      -f)   FILE_LIST_FILE_NAME="$2"
            shift
            shift ;;
      -fix_cmd) DISPLAY_FIX_COMMAND=TRUE
            shift ;;
      -filever) STABALIZE_OPTS="$STABALIZE_OPTS -filever"
            shift ;;
      -file_fixer) STABALIZE_OPTS="$STABALIZE_OPTS -file_fixer $2"
            shift
            shift ;;
      -file_names_only)
            FILE_NAMES_ONLY=TRUE
            shift ;;
      -force|-yes) STABALIZE_OPTS="$STABALIZE_OPTS -force"
            DIFF_DIR_AUTO_ANSWER="y"
            shift ;;
      -forward_only)
            FORWARD_ONLY=TRUE
            shift ;;
      -groups) STABALIZE_OPTS="$STABALIZE_OPTS -groups"
            DO_OWN_COMPARES=FALSE
            shift ;;
      -i)   IGNORE_CASE=TRUE
            shift ;;
      -ii) STABALIZE_OPTS="$STABALIZE_OPTS -i"
            shift ;;
      -identical)
            DISPLAY_IDENTICAL=TRUE
            shift ;;
      -name_fixer) NAME_FIXER="$2"
            shift
            shift ;;
      -no_excluded)
            DISPLAY_EXCLUDED=FALSE
            shift ;;
      -no_destroy) STABALIZE_OPTS="$STABALIZE_OPTS -no_destroy"
            shift ;;
      -no_identical)
            DISPLAY_IDENTICAL=FALSE
            shift ;;
      -no_different)
            DISPLAY_DIFFERENT=FALSE
            shift ;;
      -no_missing)
            DISPLAY_MISSING=FALSE
            shift ;;
      -no_prompt|-no_prompts|-no)
            STABALIZE_OPTS="$STABALIZE_OPTS -no_prompt"
            DIFF_DIR_AUTO_ANSWER="n"
            shift ;;
      -no_recompile|-no_recompiled)
            DISPLAY_RECOMPILED=FALSE
            shift ;;
      -no_stats|-no_statistics)
            DISPLAY_STATISTICS=FALSE
            shift ;;
      -owners) STABALIZE_OPTS="$STABALIZE_OPTS -owners"
            DO_OWN_COMPARES=FALSE
            shift ;;
      -perms) STABALIZE_OPTS="$STABALIZE_OPTS -perms"
            DO_OWN_COMPARES=FALSE
            shift ;;
      -quiet|-verbose)
            EZ_VERBOSITY="${LOWERCASE_ARG#-}"
            SRC_OPTS="$SRC_OPTS $1"
            shift ;;
      -stabalizer)
            STABALIZER="$2"
            shift
            shift ;;
      -stats|-statistics)
            DISPLAY_STATISTICS=TRUE
            shift ;;
      -sync)
            SYNCRONIZATION_MODE=SYNC
            shift ;;
      -tmp) TMP_DIRECTORY="$2"
            shift
            shift ;;
      -*)   if [ "$1" != "-?" ] ; then
               Usage "Invalid argument: $1"
            else
               Usage
            fi ;;
       *)   if [ "$SOURCE_DIRECTORY_NAME" = "" ] ; then
               SOURCE_DIRECTORY_NAME="$1"
            elif [ "$TARGET_DIRECTORY_NAME" = "" ] ; then
               TARGET_DIRECTORY_NAME="$1"
            else
               Usage "Too many directory names specified."
            fi
            shift ;;
    esac
done

if [ "$SOURCE_DIRECTORY_NAME" = "" -o "$TARGET_DIRECTORY_NAME" = "" ] ; then
   Usage "Must specify both a SourceDir and TargetDir." 
fi

if [ ! -d "$SOURCE_DIRECTORY_NAME" ] ; then
   echo "ERROR: Failed to find source directory $SOURCE_DIRECTORY_NAME."
   exit 1
fi

if [ ! -d "$TARGET_DIRECTORY_NAME" ] ; then
   echo "ERROR: Failed to find target directory $TARGET_DIRECTORY_NAME."
   exit 1
fi

if [ "$EXCLUDE_FILES_FILE_NAME" != "" -a \
     ! -f "$EXCLUDE_FILES_FILE_NAME" ] ; then
   touch "$EXCLUDE_FILES_FILE_NAME"
fi

if [ "$SYNCRONIZATION_MODE" = READONLY ] ; then
   STABALIZE_OPTS="$STABALIZE_OPTS -readonly"  # otherwise -sync
fi

#===========================================================================
# Determine if this file is excluded from stabalizing.
#===========================================================================
function EZIsThisFileExcluded
{
   EX_FILE_NAME="$SOURCE_DIRECTORY_NAME/$1"
   BASE_FILE_TO_CHECK="${1##*/}"

   if [ "$EXCLUDE_EXE" = TRUE ] ; then
      file "$EX_FILE_NAME" | egrep $EZ_EGREP_QUIET "executable|program database"
      if [ $? -eq 0 ] ; then
         return 0
      fi
   fi

   if [ "$EXCLUDE_FILES_FILE_NAME" = "" ] ; then
      return 1
   fi
   egrep $EZ_EGREP_QUIET "^$BASE_FILE_TO_CHECK\$" "$EXCLUDE_FILES_FILE_NAME"
   if [ $? -eq 0 ] ; then
      return 0
   fi
   return 1
}

#===========================================================================
# Determine if this directory is excluded from stabalizing.
#===========================================================================
function EZIsThisDirectoryExcluded
{
   BASE_DIR_TO_CHECK="|${1##*/}|"
   if [ "$CACHE_BASE_DIR_TO_CHECK" = "$BASE_DIR_TO_CHECK" ] ; then
      return $CACHE_DIR_EXCLUDED
   fi
   CACHE_BASE_DIR_TO_CHECK="$BASE_DIR_TO_CHECK"

   if [ "${EXCLUDE_DIRS_LIST#*$BASE_DIR_TO_CHECK}" != \
         "$EXCLUDE_DIRS_LIST" ] ; then
      CACHE_DIR_EXCLUDED=0
      return 0
   fi
   if [ "$EXCLUDE_RDIRS_LIST" != "" ] ; then
      while read EXCL_DIR
      do
         if [ "$EXCL_DIR" != "" ] ; then
            if [ "${1#*$EXCL_DIR}" != "$1" ] ; then
               CACHE_DIR_EXCLUDED=0
               return 0
            fi 
         fi
      done <<!
`echo "$EXCLUDE_RDIRS_LIST"`
!
   fi
   CACHE_DIR_EXCLUDED=1
   return 1
}

#===========================================================================
# Determine if this files extension is excluded from stabalizing.
#===========================================================================
function EZIsThisExtensionExcluded
{
   BASE_EXT_TO_CHECK="|${1##*.}|"

   if [ "${EXCLUDE_EXTS_LIST#*$BASE_EXT_TO_CHECK}" != \
         "$EXCLUDE_EXTS_LIST" ] ; then
      return 0
   fi
   return 1
}

#===========================================================================
# Add the directory to the exclude dirs file.
#===========================================================================
function EZAddExcludedDir
{
   if [ "$EXCLUDE_DIRS_FILE_NAME" = "" ] ; then
      return 1
   fi

   DIR_TO_EXCLUDE="$1"
   DIR_TO_EXCLUDE="${DIR_TO_EXCLUDE##*/}"

   cp "$EXCLUDE_DIRS_FILE_NAME" "$EXCLUDE_DIRS_FILE_NAME.fix"
   echo "$DIR_TO_EXCLUDE" >> "$EXCLUDE_DIRS_FILE_NAME.fix"
   sort "$EXCLUDE_DIRS_FILE_NAME.fix" > "$EXCLUDE_DIRS_FILE_NAME.fix1"
   mv "$EXCLUDE_DIRS_FILE_NAME.fix1" "$EXCLUDE_DIRS_FILE_NAME.fix"
   dos2unix.sh "$EXCLUDE_DIRS_FILE_NAME.fix"
   EZRun $STABALIZER -yes -r "$EXCLUDE_DIRS_FILE_NAME.fix" "$EXCLUDE_DIRS_FILE_NAME"
   EXCLUDE_DIRS_LIST="|$(cat $EXCLUDE_DIRS_FILE_NAME | tr '\n' '|')"
   CACHE_BASE_DIR_TO_CHECK=      # clear last cached exclusion.
   return 0
}

#===========================================================================
# Add the directory to the exclude rdirs file.
#===========================================================================
function EZAddExcludedRDir
{
   if [ "$EXCLUDE_RDIRS_FILE_NAME" = "" ] ; then
      return 1
   fi

   DIR_TO_EXCLUDE="$1"
   DIR_TO_EXCLUDE="${DIR_TO_EXCLUDE##*/}"

   cp "$EXCLUDE_RDIRS_FILE_NAME" "$EXCLUDE_RDIRS_FILE_NAME.fix"
   echo "$DIR_TO_EXCLUDE" >> "$EXCLUDE_RDIRS_FILE_NAME.fix"
   sort "$EXCLUDE_RDIRS_FILE_NAME.fix" > "$EXCLUDE_RDIRS_FILE_NAME.fix1"
   mv "$EXCLUDE_RDIRS_FILE_NAME.fix1" "$EXCLUDE_RDIRS_FILE_NAME.fix"
   dos2unix.sh "$EXCLUDE_RDIRS_FILE_NAME.fix"
   EZRun $STABALIZER -yes -r "$EXCLUDE_RDIRS_FILE_NAME.fix" "$EXCLUDE_RDIRS_FILE_NAME"
   EXCLUDE_RDIRS_LIST=`cat $EXCLUDE_RDIRS_FILE_NAME`
   CACHE_BASE_DIR_TO_CHECK=      # clear last cached exclusion.
   return 0
}

#===========================================================================
# Perform the requested directory comparision.
#===========================================================================

if [ "$FILE_LIST_FILE_NAME" == "" ] ; then
   TMP_OUTPUT="$TMP_DIRECTORY/ddir$$.output"
   TMP_SOURCE="$TMP_DIRECTORY/ddir$$.source"
   TMP_TARGET="$TMP_DIRECTORY/ddir$$.target"
   TMP_MERGED="$TMP_DIRECTORY/ddir$$.merged"
   SORTED_FIND_LIST="$TMP_DIRECTORY/ddir_find_sorted.$$"
   UNIQ_FIND_LIST="$TMP_DIRECTORY/ddir_find_uniq.$$"

   FIND_EXCLUDE_PATHS=
   if [ "$EXCLUDE_RDIRS_LIST" != "" ] ; then
      while read EXCL_DIR
      do
         FIND_EXCLUDE_PATHS="$FIND_EXCLUDE_PATHS -path *$EXCL_DIR -prune -o"
      done <<!
`echo "$EXCLUDE_RDIRS_LIST"`
!
   fi

   ORIG_DIR="$PWD"
   cd "$SOURCE_DIRECTORY_NAME"
   find . $FIND_EXCLUDE_PATHS -print | sed -e '/^.$/d' -e 's{^./{{' > "$TMP_SOURCE"

   if [ "$FORWARD_ONLY" = TRUE ] ; then
      echo > "$TMP_TARGET"
   else
      cd "$ORIG_DIR"
      cd "$TARGET_DIRECTORY_NAME"
      find . $FIND_EXCLUDE_PATHS -print | sed -e '/^.$/d' -e 's{^./{{' > "$TMP_TARGET"
   fi

   cd "$ORIG_DIR"
   if [ "$IGNORE_CASE" = TRUE ] ; then
      TR_CMD="tr '[:upper:]' '[:lower:]'"
      cat "$TMP_SOURCE" "$TMP_TARGET" | $TR_CMD | sort | uniq > "$TMP_MERGED"
   else
      cat "$TMP_SOURCE" "$TMP_TARGET"           | sort | uniq > "$TMP_MERGED"
   fi

   # Need descriptor 6 stuff here to support reading answers in the loop
   exec 6<< !
`cat "$TMP_MERGED"`
!
else
   # Need descriptor 6 stuff here to support reading answers in the loop
   if [ ! -f "$FILE_LIST_FILE_NAME" ] ; then
      echo "ERROR: Failed to find file $FILE_LIST_FILE_NAME."
      exit 1
   fi
   TMP_SOURCE="$FILE_LIST_FILE_NAME"  # this is for stats
   TMP_TARGET="$FILE_LIST_FILE_NAME"  # this is for stats
   TMP_MERGED="$FILE_LIST_FILE_NAME"  # this is for stats
   exec 6<< !
`cat "$FILE_LIST_FILE_NAME"`
!
fi

#===========================================================================
# View the contents of the specified directory.
#===========================================================================
function EZPerformLs
{
   LS_OBJECT_NAME="$1"
   echo
   echo "Performing: ls -asFC $LS_OBJECT_NAME"
   echo
   ls -asFC "$LS_OBJECT_NAME"
   echo
}

#===========================================================================
# Display to comparision status for the file or directory.
#===========================================================================
typeset -L12 PAD12
typeset -L9 PAD9
function EZDisplayFileStatus
{
   SOURCE_OR_TARGET="$1"
   FILE_PATH_FROM_ROOT="$2"
   STATUS_TEXT="$3"

   if [ "$STATUS_TEXT" = "Files differ" -a "$DISPLAY_DIFFERENT" = FALSE ] ; then
      return
   fi
   if [ "$STATUS_TEXT" = "Excluded" -a "$DISPLAY_EXCLUDED" = FALSE ] ; then
      return
   fi
   if [ "$STATUS_TEXT" = "Identical" -a "$DISPLAY_IDENTICAL" = FALSE ] ; then
      return
   fi
   if [ "$STATUS_TEXT" = "Dir missing" -a "$DISPLAY_MISSING" = FALSE ] ; then
      return
   fi
   if [ "$STATUS_TEXT" = "File missing" -a "$DISPLAY_MISSING" = FALSE ] ; then
      return
   fi
   if [ "$STATUS_TEXT" = "Recompiled" -a "$DISPLAY_RECOMPILED" = FALSE ] ; then
      return
   fi

   if [ "$SOURCE_OR_TARGET" = TARGET ] ; then
      if [ "$DISPLAY_DIR_TOKENS" = TRUE ] ; then
         FULL_FILE_PATH="\$DIR2/$FILE_PATH_FROM_ROOT"
      else
         FULL_FILE_PATH="$TARGET_DIRECTORY_NAME/$FILE_PATH_FROM_ROOT"
      fi
   else
      if [ "$DISPLAY_DIR_TOKENS" = TRUE ] ; then
         FULL_FILE_PATH="\$DIR1/$FILE_PATH_FROM_ROOT"
      else
         FULL_FILE_PATH="$SOURCE_DIRECTORY_NAME/$FILE_PATH_FROM_ROOT"
      fi
   fi
   if [ "$FILE_NAMES_ONLY" ] ; then
      FULL_FILE_PATH="$FILE_PATH_FROM_ROOT"
      if [ "$FULL_FILE_PATH" = "" ] ; then
         return # Root Directory
      fi
   fi

   if [ "$SYNCRONIZATION_MODE" = SYNC -a "$DIFF_DIR_AUTO_ANSWER" != n ] ; then
      # only Identical, Excluded, Recompile or questions, allow max characters
      PAD9="$STATUS_TEXT"
      PAD_STATUS="$PAD9"
   else
      PAD12="$STATUS_TEXT"
      PAD_STATUS="$PAD12"
   fi
   echo "$PAD_STATUS: $FULL_FILE_PATH"
}

if [ "$EZ_VERBOSITY" != quiet -a "$DISPLAY_DIR_TOKENS" = TRUE ] ; then
   cat <<!

=============================================================================
=== Recursively comparing directories in $SYNCRONIZATION_MODE mode.
=== DIR1=$SOURCE_DIRECTORY_NAME
=== DIR2=$TARGET_DIRECTORY_NAME
=============================================================================

!
fi

CERTIFIED_COUNT=0
CREATED_COUNT=0
DESTROYED_COUNT=0
DIFFERENT_COUNT=0
EXCLUDE_COUNT=0
IDENTICAL_COUNT=0
RECOMPILED_COUNT=0
UPDATED_COUNT=0
while read -u6 FILE_OR_DIR_NAME
do
   # Do this initial check for faster performance excluding directories.
   EZIsThisDirectoryExcluded "$FILE_OR_DIR_NAME"
   if [ $? -eq 0 ] ; then
      EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Excluded"
      let EXCLUDE_COUNT=EXCLUDE_COUNT+1
      continue
   fi

   SOURCE_FILE_OR_DIR="$SOURCE_DIRECTORY_NAME/$FILE_OR_DIR_NAME"
   TARGET_FILE_OR_DIR_NAME="$FILE_OR_DIR_NAME"
   if [ "$NAME_FIXER" != "" ] ; then
      TARGET_FILE_OR_DIR_NAME=`eval $NAME_FIXER "$FILE_OR_DIR_NAME"`
   fi
   TARGET_FILE_OR_DIR="$TARGET_DIRECTORY_NAME/$TARGET_FILE_OR_DIR_NAME"

   #echo
   #echo "FILE_OR_DIR_NAME=$FILE_OR_DIR_NAME"
   #echo "SOURCE_FILE_OR_DIR=$SOURCE_FILE_OR_DIR"
   #echo "TARGET_FILE_OR_DIR=$TARGET_FILE_OR_DIR"

   if [ -d "$SOURCE_FILE_OR_DIR" -o -d "$TARGET_FILE_OR_DIR" ] ; then
      EZIsThisDirectoryExcluded "$FILE_OR_DIR_NAME"
      if [ $? -eq 0 ] ; then
         EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Excluded"
         let EXCLUDE_COUNT=EXCLUDE_COUNT+1
         continue
      fi
      if [ -d "$SOURCE_FILE_OR_DIR" -a ! -d "$TARGET_FILE_OR_DIR" ] ; then
         if [ "$DISPLAY_DIFFS" = TRUE ] ; then
            EZPerformLs "$SOURCE_FILE_OR_DIR"
         fi
         while true
         do
            if [ "$DIFF_DIR_AUTO_ANSWER" != "" ] ; then
               if [ "$DIFF_DIR_AUTO_ANSWER" = n -o \
                    "$DIFF_DIR_AUTO_ANSWER" = i ] ; then
                  EZDisplayFileStatus "TARGET" "$FILE_OR_DIR_NAME" \
                                      "Dir missing"
               fi
               DIFF_DIR_ANSWER="$DIFF_DIR_AUTO_ANSWER"
            else
               if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
                  echo -n "Create directory $TARGET_FILE_OR_DIR <y,n,v${EXCL_DIRS_OPT}${EXCL_RDIRS_OPT}>? "
               else # READONLY
                  echo -n "Dir missing : $SOURCE_FILE_OR_DIR <i,v${EXCL_DIRS_OPT}${EXCL_RDIRS_OPT}>? "
               fi
               read DIFF_DIR_ANSWER
            fi
            if [ "$DIFF_DIR_ANSWER" = y -o "$DIFF_DIR_ANSWER" = Y ] ; then
               if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
                  mkdir -p "$TARGET_FILE_OR_DIR"
                  echo "Successfully created directory $TARGET_FILE_OR_DIR."
                  let CREATED_COUNT=CREATED_COUNT+1
                  break
               fi
            fi
            if [ "$DIFF_DIR_ANSWER" = n -o "$DIFF_DIR_ANSWER" = N -o \
                 "$DIFF_DIR_ANSWER" = i -o "$DIFF_DIR_ANSWER" = I ] ; then
               let DIFFERENT_COUNT=DIFFERENT_COUNT+1
               break
            fi
            if [ "$DIFF_DIR_ANSWER" = v -o "$DIFF_DIR_ANSWER" = V ] ; then
               EZPerformLs "$SOURCE_FILE_OR_DIR"
            fi
            if [ "$DIFF_DIR_ANSWER" = x ] ; then
               EZAddExcludedDir "$TARGET_FILE_OR_DIR"
               let EXCLUDE_COUNT=EXCLUDE_COUNT+1
               break
            fi
            if [ "$DIFF_DIR_ANSWER" = X ] ; then
               EZAddExcludedRDir "$TARGET_FILE_OR_DIR"
               let EXCLUDE_COUNT=EXCLUDE_COUNT+1
               break
            fi
         done
      elif [ ! -d "$SOURCE_FILE_OR_DIR" -a -d "$TARGET_FILE_OR_DIR" ] ; then
         if [ "$DISPLAY_DIFFS" = TRUE ] ; then
            EZPerformLs "$TARGET_FILE_OR_DIR"
         fi
         while true
         do
            if [ "$DIFF_DIR_AUTO_ANSWER" != "" ] ; then
               if [ "$DIFF_DIR_AUTO_ANSWER" = n -o \
                    "$DIFF_DIR_AUTO_ANSWER" = i ] ; then
                  EZDisplayFileStatus "SOURCE" "$FILE_OR_DIR_NAME" \
                                      "Dir missing"
               fi
               DIFF_DIR_ANSWER="$DIFF_DIR_AUTO_ANSWER"
            else
               if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
                  echo -n "Destroy directory $TARGET_FILE_OR_DIR <y,n,v${EXCL_DIRS_OPT}${EXCL_RDIRS_OPT}>? "
               else # READONLY
                  echo -n "Dir missing : $SOURCE_FILE_OR_DIR <i,v${EXCL_DIRS_OPT}${EXCL_RDIRS_OPT}>? "
               fi
               read DIFF_DIR_ANSWER
            fi
            if [ "$DIFF_DIR_ANSWER" = y ] ; then
               if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
                  #rm -rf "$TARGET_FILE_OR_DIR"
                  rmdir "$TARGET_FILE_OR_DIR"
                  if [ $? -eq 0 ] ; then
                     echo "Successfully removed directory $TARGET_FILE_OR_DIR."
                     let DESTROYED_COUNT=DESTROYED_COUNT+1
                  else
                     echo -n "ERROR: Failed to destroy directory "
                     echo    "$TARGET_FILE_OR_DIR."
                  fi
                  break
               fi
            fi
            if [ "$DIFF_DIR_ANSWER" = Y ] ; then
               if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
                  rm -rf "$TARGET_FILE_OR_DIR"
                  if [ $? -eq 0 ] ; then
                     echo "Successfully removed directory $TARGET_FILE_OR_DIR."
                     let DESTROYED_COUNT=DESTROYED_COUNT+1
                  else
                     echo -n "ERROR: Failed to destroy directory "
                     echo    "$TARGET_FILE_OR_DIR."
                  fi
                  break
               fi
            fi
            if [ "$DIFF_DIR_ANSWER" = n -o "$DIFF_DIR_ANSWER" = N -o \
                 "$DIFF_DIR_ANSWER" = i -o "$DIFF_DIR_ANSWER" = I ] ; then
               let DIFFERENT_COUNT=DIFFERENT_COUNT+1
               break
            fi
            if [ "$DIFF_DIR_ANSWER" = v -o "$DIFF_DIR_ANSWER" = V ] ; then
               EZPerformLs "$TARGET_FILE_OR_DIR"
            fi
            if [ "$DIFF_DIR_ANSWER" = x ] ; then
               EZAddExcludedDir "$TARGET_FILE_OR_DIR"
               let EXCLUDE_COUNT=EXCLUDE_COUNT+1
               break
            fi
            if [ "$DIFF_DIR_ANSWER" = X ] ; then
               EZAddExcludedRDir "$TARGET_FILE_OR_DIR"
               let EXCLUDE_COUNT=EXCLUDE_COUNT+1
               break
            fi
         done
      else
         EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Identical"
         let IDENTICAL_COUNT=IDENTICAL_COUNT+1
      fi
   else
      EZIsThisDirectoryExcluded "${FILE_OR_DIR_NAME%/*}"
      if [ $? -eq 0 ] ; then
         EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Excluded"
         let EXCLUDE_COUNT=EXCLUDE_COUNT+1
         continue
      fi

      EZIsThisExtensionExcluded "$FILE_OR_DIR_NAME"
      if [ $? -eq 0 ] ; then
         EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Excluded"
         let EXCLUDE_COUNT=EXCLUDE_COUNT+1
         continue
      fi

      # Do our own exclusion check for faster performance
      EZIsThisFileExcluded "$FILE_OR_DIR_NAME"
      if [ $? -eq 0 ] ; then
         EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Excluded"
         let EXCLUDE_COUNT=EXCLUDE_COUNT+1
         continue
      fi

      # Do our own comparision check for faster performance
      if [ -f "$SOURCE_FILE_OR_DIR" -a -f "$TARGET_FILE_OR_DIR" -a \
           "$DO_OWN_COMPARES" = TRUE ] ; then
         if [ "$USE_DIFF_EXE" = TRUE ] ; then
            EZRun -output_file "$TMP_OUTPUT" \
                 diff "$SOURCE_FILE_OR_DIR" "$TARGET_FILE_OR_DIR" 
            DIFF_CMP_EXIT=$?
            if [ $DIFF_CMP_EXIT -ne 0 ] ; then
               egrep $EZ_EGREP_QUIET "is binary" "$TMP_OUTPUT"
               if [ $? -eq 0 ] ; then # even if want diff, use cmp if binary
                  EZRun cmp -s "$SOURCE_FILE_OR_DIR" "$TARGET_FILE_OR_DIR"
                  DIFF_CMP_EXIT=$?
               fi
            fi
         else
            EZRun cmp -s "$SOURCE_FILE_OR_DIR" "$TARGET_FILE_OR_DIR"
            DIFF_CMP_EXIT=$?
         fi
         if [ $DIFF_CMP_EXIT -eq 0 ] ; then
            EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Identical"
            let IDENTICAL_COUNT=IDENTICAL_COUNT+1
            continue
         else
            if [ "$DISPLAY_DIFFERENT" = FALSE ] ; then
               continue
            fi
         fi
      fi

      # use src stabalize for all the really hard stuff
      EZRun $STABALIZER $STABALIZE_OPTS \
          -t Certified "" -t Excluded "" -t Recompiled "" -t Unchanged "" \
          "$SOURCE_FILE_OR_DIR" "$TARGET_FILE_OR_DIR"
      STABALIZE_EXIT_CODE=$?
      # 0) Matched  1) Created     2) Updated     3) Destroyed
      # 4) Excluded 5) Not Updated 6) Recompiled  7) Certified
      if [ $STABALIZE_EXIT_CODE -eq 0 ] ; then
         EZDisplayFileStatus "TARGET" "$TARGET_FILE_OR_DIR_NAME" "Identical"
         let IDENTICAL_COUNT=IDENTICAL_COUNT+1
      elif [ $STABALIZE_EXIT_CODE -eq 1 ] ; then
         let CREATED_COUNT=CREATED_COUNT+1
      elif [ $STABALIZE_EXIT_CODE -eq 2 ] ; then
         let UPDATED_COUNT=UPDATED_COUNT+1
      elif [ $STABALIZE_EXIT_CODE -eq 3 ] ; then
         let DESTROYED_COUNT=DESTROYED_COUNT+1
      elif [ $STABALIZE_EXIT_CODE -eq 4 ] ; then
         EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Excluded"
         let EXCLUDE_COUNT=EXCLUDE_COUNT+1
      elif [ $STABALIZE_EXIT_CODE -eq 5 ] ; then
         if [ "$DIFF_DIR_AUTO_ANSWER" = n -o \
              "$DIFF_DIR_AUTO_ANSWER" = i ] ; then # echo for src stabalize
            if [ ! -r "$SOURCE_FILE_OR_DIR" ] ; then
               EZDisplayFileStatus "SOURCE" "$FILE_OR_DIR_NAME" \
                                   "File missing"
            elif [ ! -r "$TARGET_FILE_OR_DIR" ] ; then
               EZDisplayFileStatus "TARGET" "$FILE_OR_DIR_NAME" \
                                   "File missing"
            else
               EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Files differ"
            fi
         fi
         if [ "$DISPLAY_FIX_COMMAND" = TRUE ] ; then
            echo "Certify with: $STABALIZER $STABALIZE_OPTS $SOURCE_FILE_OR_DIR $TARGET_FILE_OR_DIR" | sed 's/ -no_prompt//'
            echo
         fi
         let DIFFERENT_COUNT=DIFFERENT_COUNT+1
      elif [ $STABALIZE_EXIT_CODE -eq 6 ] ; then
         let RECOMPILED_COUNT=RECOMPILED_COUNT+1
         EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Recompiled"
      elif [ $STABALIZE_EXIT_CODE -eq 7 ] ; then
         let CERTIFIED_COUNT=CERTIFIED_COUNT+1
         echo -n "           " # indent these under the Certified reason
         if [ ! -r "$SOURCE_FILE_OR_DIR" ] ; then
            EZDisplayFileStatus "SOURCE" "$FILE_OR_DIR_NAME" \
                                "File missing"
         elif [ ! -r "$TARGET_FILE_OR_DIR" ] ; then
            EZDisplayFileStatus "TARGET" "$FILE_OR_DIR_NAME" \
                                "File missing"
         else
            EZDisplayFileStatus "BOTH" "$FILE_OR_DIR_NAME" "Files differ"
         fi
         echo
      fi
   fi
done

exec 6<&-   # Close 6

SOURCE_COUNT=`wc -l "$TMP_SOURCE" | awk '{print $1}'`
TARGET_COUNT=`wc -l "$TMP_TARGET" | awk '{print $1}'`
MERGED_COUNT=`wc -l "$TMP_MERGED" | awk '{print $1}'`
let TOTAL_COUNT=SOURCE_COUNT+TARGET_COUNT
let TOTAL_MATCHED=TOTAL_COUNT-MERGED_COUNT

if [ "$DISPLAY_STATISTICS" = TRUE ] ; then
   printf "\n"
   printf "%7d Files/dirs found in $SOURCE_DIRECTORY_NAME.\n" $SOURCE_COUNT
   printf "%7d Files/dirs found in $TARGET_DIRECTORY_NAME.\n" $TARGET_COUNT
   printf "%7d Files/dirs found in both the source and target directories.\n" \
                                                           $TOTAL_COUNT
   printf "%7d Unique file/dir names in both the source and target directories.\n" \
                                                           $MERGED_COUNT
   printf "%7d File/dir name matches between the source and target directories.\n" \
                                                           $TOTAL_MATCHED
fi

if [ "$EZ_VERBOSITY" != quiet ] ; then
   echo
   echo -n "======================================="
   echo    "========================================"
   if [ "$EXCLUDE_COUNT" != "0" ] ; then
      printf "=== %7d Excluded\n" $EXCLUDE_COUNT
   fi
   if [ "$CREATED_COUNT" != "0" ] ; then
      printf "=== %7d Created\n" $CREATED_COUNT
   fi
   if [ "$DESTROYED_COUNT" != "0" ] ; then
      printf "=== %7d Destroyed\n" $DESTROYED_COUNT
   fi
   if [ "$UPDATED_COUNT" != "0" ] ; then
      printf "=== %7d Updated\n" $UPDATED_COUNT
   fi
   if [ "$RECOMPILED_COUNT" != "0" ] ; then
      printf "=== %7d Recompiled\n" $RECOMPILED_COUNT
   fi
   if [ "$CERTIFIED_COUNT" != "0" ] ; then
      printf "=== %7d Certified\n" $CERTIFIED_COUNT
   fi
   if [ "$DIFFERENT_COUNT" != "0" ] ; then
      printf "=== %7d Different\n" $DIFFERENT_COUNT
   fi
   if [ "$IDENTICAL_COUNT" != "0" ] ; then
      printf "=== %7d Identical\n" $IDENTICAL_COUNT
   fi
   printf "=== %7d Total files/directories\n" $MERGED_COUNT
   echo -n "======================================="
   echo    "========================================"
   echo
fi

rm -f "$TMP_DIRECTORY/ddir$$."*

exit 0
