#===========================================================================
#  Filename: DiffFiles.ksh
#    Author: Craig Schneider
# Orig Date: 04/18/13
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Diff two files using various comparision methods.
#===========================================================================

# TODO: Need command line argument for tail lines of the file to diff.
# TODO: Default $COL_DATA_DIR directory to under $TMPDIR and remove when done

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: DiffFiles [-Cat] [-ColDataDir DataDirectory] [-ColSeparator Separator]
                 [-Diffs] [-Dos2Unix] [-Head HeadLines] [-Quiet] [-StopOnDiff]
                 [-Tmp TempDir] SourceFile TargetFile

         -Cat: Cat the content of the column files with differences.
  -ColDataDir: The path to the col data directory.
-ColSeparator: The separator character used betweeen the columns (default ^).
       -Diffs: Display the differences in content of the columns.
    -Dos2Unix: dos2unix versions of files are diffed.
        -Head: Specifies the number of head lines to be diffed.
  -StopOnDiff: Stop showing column diffs after the first column difference.
       -Quiet: Suppress non critical messages about operations being performed.
   SourceFile: The source file to be used for the diff.
         -Tmp: Specifies the temporary directory to use.
   TargetFile: The target file to be used for the diff.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = "-?" ] ; then
   Usage
fi
if [ "$#" -eq 0 ] ; then
   Usage
fi

EXIT_CODE=0

CAT_FILES=FALSE
COL_DATA_DIR="ColData"
COL_SEP="^"
DIFF_FILES=FALSE
DOS2UNIX_FILES=FALSE
HEADER_LINE_COUNT=0
HEAD_LINES=
SOURCE_FILE_NAME=
STOP_ON_DIFF=FALSE
TARGET_FILE_NAME=
EZ_VERBOSITY=
DIFF_TMP=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -cat)
          CAT_FILES=TRUE
          shift ;;
      -coldatadir)
          COL_DATA_DIR="$2"
          shift
          shift ;;
      -colseparator)
          COL_SEP="$2"
          shift
          shift ;;
      -diffs)
          DIFF_FILES=TRUE
          shift ;;
      -dos2unix)
          DOS2UNIX_FILES=TRUE
          shift ;;
      -stopondiff)
          STOP_ON_DIFF=TRUE
          shift ;;
      -head)
          HEAD_LINES="-"
          HEAD_LINES="${HEAD_LINES}$2"
          shift
          shift ;;
      -tmp)
          DIFF_TMP="$2"
          shift
          shift ;;
      -quiet)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) if [ "$SOURCE_FILE_NAME" = "" ] ; then
             SOURCE_FILE_NAME="$1"
          elif [ "$TARGET_FILE_NAME" = "" ] ; then
             TARGET_FILE_NAME="$1"
          else
             Usage "Too many file names specified."
          fi
          shift ;;
    esac
done

if [ "$SOURCE_FILE_NAME" = "" -o "$TARGET_FILE_NAME" = "" ] ; then
   Usage "Must specify both a SourceFile and TargetFile."
fi

SOURCE_FILE_NAME_CMP="$SOURCE_FILE_NAME"
TARGET_FILE_NAME_CMP="$TARGET_FILE_NAME"

if [ "$HEAD_LINES" != "" ] ; then
   if [ "$DIFF_TMP" != "" ] ; then
      SOURCE_NAME="${SOURCE_FILE_NAME_CMP##*/}"
      head $HEAD_LINES "$SOURCE_FILE_NAME_CMP" > "$DIFF_TMP/$SOURCE_NAME.headS"
      SOURCE_FILE_NAME_CMP="$DIFF_TMP/$SOURCE_NAME.headS"
      TARGET_NAME="${TARGET_FILE_NAME_CMP##*/}"
      head $HEAD_LINES "$TARGET_FILE_NAME_CMP" > "$DIFF_TMP/$TARGET_NAME.headT"
      TARGET_FILE_NAME_CMP="$DIFF_TMP/$TARGET_NAME.headT"
   else
      head $HEAD_LINES "$SOURCE_FILE_NAME_CMP" > "$SOURCE_FILE_NAME_CMP".head
      SOURCE_FILE_NAME_CMP="$SOURCE_FILE_NAME_CMP".head 
      head $HEAD_LINES "$TARGET_FILE_NAME_CMP" > "$TARGET_FILE_NAME_CMP".head
      TARGET_FILE_NAME_CMP="$TARGET_FILE_NAME_CMP".head
   fi
fi
FILES_IDENTICAL=FALSE

DIFF_OUTPUT_FILE_NAME="$TMPDIR/DiffFiles.ksh.$$"
diff "$SOURCE_FILE_NAME_CMP" "$TARGET_FILE_NAME_CMP" > "$DIFF_OUTPUT_FILE_NAME"
if [ $? -eq 0 ] ; then
   if [ "$EZ_VERBOSITY" != "quiet" ] ; then
      echo "Successfully detected $SOURCE_FILE_NAME_CMP is identical to \c"
      echo "$TARGET_FILE_NAME_CMP."
   fi
   rm -f "$DIFF_OUTPUT_FILE_NAME"
   exit 0
fi

rm -f "$DIFF_OUTPUT_FILE_NAME"

mkdir -p "$COL_DATA_DIR"
if [ "$DOS2UNIX_FILES" = "TRUE" ] ; then
   dos2unix.sh "$SOURCE_FILE_NAME_CMP"
   dos2unix.sh "$TARGET_FILE_NAME_CMP"
fi
TARGET_COLS=`tail -1 "$TARGET_FILE_NAME_CMP" | tr "$COL_SEP" '\n' | \
             wc -l | awk '{ print $1 }'`
SOURCE_COLS=`tail -1 "$SOURCE_FILE_NAME_CMP" | tr "$COL_SEP" '\n' | \
             wc -l | awk '{ print $1 }'`
TARGET_ROWS=`wc -l "$TARGET_FILE_NAME_CMP" | awk '{ print $1 }'`
let TARGET_ROWS=TARGET_ROWS-HEADER_LINE_COUNT
SOURCE_ROWS=`wc -l "$SOURCE_FILE_NAME_CMP" | awk '{ print $1 }'`
let SOURCE_ROWS=SOURCE_ROWS-HEADER_LINE_COUNT

if [ $SOURCE_COLS -ne $TARGET_COLS ] ; then
   echo "ERROR: Detected $SOURCE_COLS source columns but \c"
   echo "$TARGET_COLS target columns."
fi
if [ $SOURCE_ROWS -ne $TARGET_ROWS ] ; then
   echo "ERROR: Detected $SOURCE_ROWS source rows but \c"
   echo "$TARGET_ROWS target rows."
fi
TOTAL_COLS=$SOURCE_COLS

TOTAL_COLUMNS_GOOD=0
TOTAL_COLUMNS_NO_DATA=0
TOTAL_COLUMNS_DIFF=0
COL=1
while [ $COL -le $TOTAL_COLS ]
do
   cut -f $COL -d$COL_SEP "$TARGET_FILE_NAME_CMP" \
                          > "$COL_DATA_DIR/Old$COL.dat"
   cut -f $COL -d$COL_SEP "$SOURCE_FILE_NAME_CMP" \
                          > "$COL_DATA_DIR/New$COL.dat"
   diff "$COL_DATA_DIR/New$COL.dat" "$COL_DATA_DIR/Old$COL.dat" \
                                  > "$COL_DATA_DIR/Diffs$COL.dat"
   if [ $? -eq 0 ] ; then
      if [ $HEADER_LINE_COUNT -eq 1 ] ; then
         cat "$COL_DATA_DIR/New$COL.dat" | sed -e '1d' -e '/^$/d' \
           > "$COL_DATA_DIR/NewContent.dat"
      else
         cat "$COL_DATA_DIR/New$COL.dat" | sed -e '/^$/d' \
           > "$COL_DATA_DIR/NewContent.dat"
      fi
      CONTENT_LINES=`wc -l "$COL_DATA_DIR/NewContent.dat" | awk '{ print $1 }'`
      if [ $CONTENT_LINES -eq 0 ] ; then
         echo "Successfully detected column #$COL matches but has no data."
         let TOTAL_COLUMNS_NO_DATA=TOTAL_COLUMNS_NO_DATA+1
      else
         echo "Successfully detected column #$COL matches the target values."
         let TOTAL_COLUMNS_GOOD=TOTAL_COLUMNS_GOOD+1
      fi
   else
      echo "Error found differences in column #$COL."
      let TOTAL_COLUMNS_DIFF=TOTAL_COLUMNS_DIFF+1
      if [ "$CAT_FILES" = TRUE ] ; then
         echo "Line Num: Source File (New$COL.dat) / Target File (Old$COL.dat)\n"
      
         grep -n '[a-~]*' "$COL_DATA_DIR/New$COL.dat"  # Shows line numbers
         echo -n "-------------------------------------\c"
         echo    "-------------------------------------"
         grep -n '[a-~]*' "$COL_DATA_DIR/Old$COL.dat"  # Shows line numbers
      fi
      if [ "$DIFF_FILES" = TRUE ] ; then
         cat "$COL_DATA_DIR/Diffs$COL.dat"
      fi
      if [ "$STOP_ON_DIFF" = TRUE ] ; then
         exit 1
      fi
   fi
   rm -f "$COL_DATA_DIR/ColDiffs$COL.dat"

   let COL=COL+1
done

echo
echo "Successfully processed $SOURCE_ROWS rows of data."
echo "Successfully detected $TOTAL_COLUMNS_GOOD matching columns."
echo "Successfully detected $TOTAL_COLUMNS_NO_DATA matching columns no data."
echo "Successfully detected $TOTAL_COLUMNS_DIFF columns containing diffs."
if [ $SOURCE_COLS -ne $TARGET_COLS ] ; then
   echo "ERROR: Detected $SOURCE_COLS source columns but \c"
   echo "$TARGET_COLS target columns."
fi
if [ $SOURCE_ROWS -ne $TARGET_ROWS ] ; then
   echo "ERROR: Detected $SOURCE_ROWS source rows but \c"
   echo "$TARGET_ROWS target rows."
fi
exit 1
