#===========================================================================
#  Filename: PerfEvtFileMerger.ksh
#    Author: Craig Schneider
# Orig Date: 06/03/14
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Performs a merge operation of several PerfEvt.csv files.
#===========================================================================

. $EZ_SHARED_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: PerfEvtFileMerger -Mode Mode -OutputFileName OutputFileName 
                         [-FileNameFilter FileNameFilter]
                         [-FileNameFilter1 FileNameFilter1]
                         [-FileNameFilterOut FileNameFilterOut]
                         [-OnlyLastFile]
                         [-RequestCategoryFilter RequestCategoryFilter]
                         [-RequestTopicFilter RequestTopicFilter]
                         [-SortMode SortMode]
                         [-UnitsFilter UnitsFilter]

       FileNameFilter: The egrep expression of file names to include.
      FileNameFilter1: The egrep expression of file names to include.
    FileNameFilterOut: The egrep expression of file names to exclude.
         OnlyLastFile: Secifies to only use the most recent file from the list.
                 Mode: The major mode of operation, the supported modes are
                       CodeCoverage and Parse.
       OutputFileName: The name of the merged output file to create.
RequestCategoryFilter: The egrep expression for filtering the request topic.
   RequestTopicFilter: The egrep expression for filtering the request topic.
             SortMode: Sort mode to use for the merged output file, the     
                       supported modes are None, Time and Units (default Time).
          UnitsFilter: The egrep expression for filtering the units.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================

FILE_NAME_FILTER=
FILE_NAME_FILTER_1=
FILE_NAME_FILTER_OUT=
INTERNAL_FILE_NAME_FILTER=
MAX_DATA_POINTS=2000
ONLY_LAST_FILE=FALSE
OUTPUT_FILE_NAME=
PERF_STATS_ROOT=
REQUEST_CATEGORY_FILTER=
REQUEST_TOPIC_FILTER=
UNITS_FILTER=
typeset -u MERGER_MODE=
typeset -u SORT_MODE=TIME

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -filenamefilter)
          FILE_NAME_FILTER="$2"
          shift
          shift ;;
      -filenamefilter1)
          FILE_NAME_FILTER_1="$2"
          shift
          shift ;;
      -filenamefilterout)
          FILE_NAME_FILTER_OUT="$2"
          shift
          shift ;;
      -mode)
          MERGER_MODE="$2"
          shift
          shift ;;
      -onlylastfile)
          ONLY_LAST_FILE="TRUE"
          shift ;;
      -outputfilename)
          OUTPUT_FILE_NAME="$2"
          shift
          shift ;;
      -requestcategoryfilter)
          REQUEST_CATEGORY_FILTER="$2"
          shift
          shift ;;
      -requesttopicfilter)
          REQUEST_TOPIC_FILTER="$2"
          shift
          shift ;;
      -sortmode)
          SORT_MODE="$2"
          shift
          shift ;;
      -unitsfilter)
          UNITS_FILTER="$2"
          shift
          shift ;;
       *) if [ "$1" = "-?" ] ; then
             Usage
          fi
          shift ;;
    esac
done

TEMP_DATA_FILE="$TMPDIR/PerfEvtFileMerger.$$"
rm -f "$TEMP_DATA_FILE"

if [ "$MERGER_MODE" = CODECOVERAGE ] ; then
   PERF_STATS_ROOT="P:/Development/Archive/PerfStatsCodeCoverage"
   INTERNAL_FILE_NAME_FILTER=".C4Coverage."
elif [ "$MERGER_MODE" = PARSE ] ; then
   PERF_STATS_ROOT="C:/LocalFiles/PerfStats"
   INTERNAL_FILE_NAME_FILTER=".FileParse."
else
   Usage "Must specify either -Mode CodeCoverage or -Mode Parse."
fi

if [ "$OUTPUT_FILE_NAME" = "" ] ; then
   Usage "Must specify -OutputFileName OutputFileName."
fi

if [ ! -d "$PERF_STATS_ROOT" ] ; then
   echo "ERROR: Failed to find perf stats directory $PERF_STATS_ROOT."
   exit 1
fi

if [ "$PERF_CHARTS_ROOT" = "" ] ; then
   PERF_CHARTS_ROOT="${PERF_STATS_ROOT}Charts"
fi

if [ ! -d "$PERF_CHARTS_ROOT" ] ; then
   echo "ERROR: Failed to find perf charts directory $PERF_CHARTS_ROOT."
   exit 1
fi

########################################################################
# Setup All Unspecified Filters
########################################################################
if [ "$FILE_NAME_FILTER_OUT" = "" ] ; then
   FILE_NAME_FILTER_OUT="XYZXYZ"
fi
if [ "$FILE_NAME_FILTER" = "" ] ; then
   FILE_NAME_FILTER="csv"
fi
if [ "$FILE_NAME_FILTER_1" = "" ] ; then
   FILE_NAME_FILTER_1="csv"
fi
if [ "$UNITS_FILTER" = "" ] ; then
   UNITS_FILTER=","
fi
if [ "$REQUEST_CATEGORY_FILTER" = "" ] ; then
   REQUEST_CATEGORY_FILTER=","
fi
if [ "$REQUEST_TOPIC_FILTER" = "" ] ; then
   REQUEST_TOPIC_FILTER=","
fi

########################################################################
# Create the single header line of the file
########################################################################
OUTPUT_FILE_FULL_NAME="$PERF_CHARTS_ROOT/$OUTPUT_FILE_NAME"
CANDIDATE_FILE_NAMES=`ls \
      $PERF_STATS_ROOT/*${INTERNAL_FILE_NAME_FILTER}*.PerfEvt.csv | \
      egrep "$FILE_NAME_FILTER" | \
      egrep "$FILE_NAME_FILTER_1" | \
      egrep -v "$FILE_NAME_FILTER_OUT"`

LAST_FILE_NAME=`echo "$CANDIDATE_FILE_NAMES" | tail -1`

egrep StartDateTime $LAST_FILE_NAME > "$OUTPUT_FILE_FULL_NAME"
if [ "$?" -ne 0 ] ; then
   echo -n "ERROR: Failed merging stats into file $OUTPUT_FILE_FULL_NAME "
   echo    "it must be in use."
   exit 1
fi

if [ "$ONLY_LAST_FILE" = TRUE ] ; then
   CANDIDATE_FILE_NAMES="$LAST_FILE_NAME"
fi

egrep -h "$UNITS_FILTER" $CANDIDATE_FILE_NAMES | \
              egrep "$REQUEST_CATEGORY_FILTER" | \
              egrep "$REQUEST_TOPIC_FILTER" >> "$TEMP_DATA_FILE"

TAIL_ARGS="-$MAX_DATA_POINTS"
if [ "$SORT_MODE" = UNITS ] ; then
   sort -n -k 4 -t , \
        "$TEMP_DATA_FILE" | tail $TAIL_ARGS >> "$OUTPUT_FILE_FULL_NAME"
elif [ "$SORT_MODE" = TIME ] ; then
   sort "$TEMP_DATA_FILE" | tail $TAIL_ARGS >> "$OUTPUT_FILE_FULL_NAME"
else
   cat  "$TEMP_DATA_FILE" | tail $TAIL_ARGS >> "$OUTPUT_FILE_FULL_NAME"
fi

rm -f "$TEMP_DATA_FILE"

echo "Successfully merged stats into $OUTPUT_FILE_FULL_NAME."
exit 0
