#===========================================================================
#  Filename: fam_subscribe.ksh
#    Author: Craig Schneider
# Orig Date: 07/10/13
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Subscribe this computer to a source control family.
#===========================================================================

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: fam subscribe [-dryrun] [-family FamilyName] [-host HostName] [-status] 
                     [-quiet | -verbose]

     -dryrun: Display the commands to be executed, without executing them.
     -family: Specifies the name of the source control family to subscribe.
       -host: Specifies the name of the host to subscribe.
     -status: Just return the subscription status.
      -quiet: Suppress non critical messages about operations being performed.
    -verbose: Display verbose messages about operations being performed.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi

EXIT_CODE=0
EZ_DRYRUN=FALSE
EZ_VERBOSITY=
FAMILY_NAME=
HOST_NAME=
STATUS_MODE=FALSE
UNSUBSCRIBE_MODE=FALSE
FEATURE_ARGS="-name Subscribed"

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
     -dryrun) EZ_DRYRUN=TRUE
              FEATURE_ARGS="$FEATURE_ARGS $1"
              shift ;;
     -family) FAMILY_NAME="$2"
              FEATURE_ARGS="$FEATURE_ARGS $1 $2"
              shift
              shift ;;
       -host) HOST_NAME="$2"
              FEATURE_ARGS="$FEATURE_ARGS $1 $2"
              shift
              shift ;;
     -status) STATUS_MODE=TRUE
              FEATURE_ARGS="$FEATURE_ARGS $1"
              shift ;;
  -subscribe) UNSUBSCRIBE_MODE=FALSE
              shift ;;
-unsubscribe) UNSUBSCRIBE_MODE=TRUE
              shift ;;
-quiet|-verbose)
              EZ_VERBOSITY="${LOWERCASE_ARG#-}"
              FEATURE_ARGS="$FEATURE_ARGS $1"
              shift ;;
           *) if [ "$1" != "-?" ] ; then
                 Usage "Invalid argument: $1"
              else
                 Usage
              fi ;;
    esac
done

if [ "$HOST_NAME" = "" ] ; then
   HOST_NAME="$EZ_COMPUTERNAME"
fi

if [ "$EZ_ACTIVE_RELEASE" != "working" ] ; then
   echo -n "ERROR: Subscriptions are only supported for the working stream "
   echo    "not $EZ_ACTIVE_RELEASE."
   exit 1
fi

#===========================================================================
# Perform the requested operations.
#===========================================================================

if [ "$FAMILY_NAME" = "" ] ; then
   echo
   PS3="Select Family Name: "
   SAVE_IFS="$IFS"
   IFS="
"
   select FAMILY_NAME_SELECT in `fam.sh list`
   do
      if [ "$FAMILY_NAME_SELECT" != "" ] ; then
         FAMILY_NAME="${FAMILY_NAME_SELECT%% *}"
         break
      fi
   done
   IFS="$SAVE_IFS"
   FEATURE_ARGS="$FEATURE_ARGS -family $FAMILY_NAME"
fi

if [ "$FAMILY_NAME" = "" ] ; then
   echo
   echo "ERROR: No family has been specified."
   exit 1
fi

if [ "$STATUS_MODE" = TRUE ] ; then
   fam.sh feature $FEATURE_ARGS
   EXIT_CODE=$?
else
   TOP_DIR_KSH=`Path2Ksh.ksh "${EZ_FAMILYTREE_ROOT%/*}"`
   cd "$TOP_DIR_KSH"
   if [ "$UNSUBSCRIBE_MODE" = TRUE ] ; then
      if [ "$FAMILY_NAME" = "FamilyTree" ] ; then
         echo "ERROR: Unsubscribing from $FAMILY_NAME is not allowed."
         exit 1
      fi

      SVN_OUTPUT="$TMPDIR/fam_sub_svn.$$"
      svn update --set-depth exclude $FAMILY_NAME 2>&1 > "$SVN_OUTPUT"
      if [ $? -ne 0 ] ; then
         egrep $EZ_EGREP_QUIET ' was not found.' "$SVN_OUTPUT"
         EGREP_RET=$?
         if [ "$EGREP_RET" -eq 0 ] ; then
            cat "$SVN_OUTPUT"
            rm -f "$SVN_OUTPUT"
            echo "ERROR: Failed excluding $FAMILY_NAME from local repository."
            exit 1
         else
            if [ "$EZ_VERBOSITY" != quiet ] ; then
               echo -n "Warning: Failed to find local repository \c"
               echo    "directory $FAMILY_NAME."
            fi
            rm -f "$SVN_OUTPUT"
         fi
      else
         cat "$SVN_OUTPUT"
         rm -f "$SVN_OUTPUT"
      fi
      if [ -d $FAMILY_NAME ] ; then
         echo "ERROR: Detected un-committed changes in family $FAMILY_NAME."
         rm -rf $FAMILY_NAME.Unsubscribed
         mv $FAMILY_NAME $FAMILY_NAME.Unsubscribed
         if [ $? -ne 0 ] ; then
            echo "ERROR: Failed moving to $PWD/$FAMILY_NAME.Unsubscribed."
            exit 1
         fi
         echo "ERROR: Moved to $PWD/$FAMILY_NAME.Unsubscribed."
      fi
      FEATURE_ARGS="$FEATURE_ARGS -delete"
   else
      svn update $FAMILY_NAME | sed -e '/^Updating /d' \
                                    -e '/^At revision /d'
      FEATURE_ARGS="$FEATURE_ARGS -update"
   fi

   fam.sh feature $FEATURE_ARGS
   EXIT_CODE=$?

   if [ "$UNSUBSCRIBE_MODE" = TRUE ] ; then
      fam.sh feature -delete -name BuildRev -family $FAMILY_NAME \
                      -host $HOST_NAME
   fi

   if [ "$EZ_VERBOSITY" != quiet ] ; then
      echo
      echo "Be sure to either start a new KSH or execute the dot command!!!"
   fi
fi

exit $EXIT_CODE
