#===========================================================================
#  Filename: mk_lists.ksh
#    Author: Craig Schneider
# Orig Date: 02/09/97
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Generates pre-indexed lists of files and directories.
#===========================================================================

. $EZ_SHARED_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: $0 [-all] [-d] [-f] [-v] [-family FamilyName]

      -all: Make all family lists (default)
        -d: Make the directory family list (dir_list.dat)
        -f: Make the file family list (src_list.dat)
   -family: Make the list for the specified family only
        -v: Verbose mode shows the list of differences

!
   exit 1
}

FILE_TYPE_LIST="directories files"
FAMILY_LIST="$EZ_FAM_ROOTS"
STABALIZE_ANSWER="-yes"

while [ "$#" -ne 0 ]
do
  case $1 in
  -all) FILE_TYPE_LIST="directories files"
        shift ;;
    -d) FILE_TYPE_LIST=directories
        shift ;;
    -family) FAMILY_LIST="$2"
        if [ ! -d "$FAMILY_LIST" ] ; then
           echo -n "ERROR: Failed to find directory $FAMILY_LIST expecting "
           echo    "something like C:/Source/FamilyTree."
           exit 1
        fi
        shift
        shift ;;
    -f) FILE_TYPE_LIST=files
        shift ;;
    -v) STABALIZE_ANSWER=
        shift ;;
     *) if [ "$1" != "-?" ] ; then
           Usage "Invalid argument: $1"
        else
           Usage
        fi ;;
  esac
done

FILE_EGREP_LIST="C4BaseCoreTest|CustomerData|Upgrades|UpgX|TestData|vData|VersionOneObjectModel"
DIR_EGREP_LIST="/bin\$|/bin/|/Bin\$|ClientBin|/com/|/com\$|/C4NugetStaged|/CoreWebServices|/Debug|/dist\$|/dist/|DNetBin|DNetCache|IgniteUI|JavaBin|jquery|Knockout|/lib\$|/node_modules|/nugets|/obj\$|/obj/x86\$|/RPMSrc/|/packages/|/packages\$|Properties|QUnit|SQLiteSystemData|Staged|.svn|/TestData|/TestResults|/vData\$|/.vs"

#===========================================================================
# Make the desired list file across all configured families.
#===========================================================================
function MakeFamilyList
{
   LIST_MODE=$1

   WARNINGS_ON=TRUE
   if [ "$LIST_MODE" = "files" ] ; then
      PRETTY_NAME="Source File"
      THE_LIST_FILE=src_list.dat
   elif [ "$LIST_MODE" = "directories" ] ; then
      PRETTY_NAME=Directory
      THE_LIST_FILE=dir_list.dat
   fi

   typeset -u ROOT_VAR
   typeset -R4 NUMBER
   for ROOT_DIR in $FAMILY_LIST
   do
      THE_LIST_DIR="$ROOT_DIR/Env"
      LIST_FILE="$THE_LIST_DIR/$THE_LIST_FILE"
      if [ ! -f $LIST_FILE ] ; then
         if [ "$WARNINGS_ON" = TRUE ] ; then
            echo "Warning: $PRETTY_NAME list $LIST_FILE not found!"
         fi
         continue
      fi

      TMP_FILE=$TMPDIR/mk_list.$$.tmp
      FAMILY_DIR=${ROOT_DIR##*/}
      ROOT_VAR="EZ_${FAMILY_DIR}_ROOT"
      cd $ROOT_DIR

      if [ "$LIST_MODE" = "files" ] ; then
         if [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
            ss directory $EZ_RELEASE_ROOT/$FAMILY_DIR -R $EZ_VSS_OPTIONS | \
               awk ' \
               BEGIN { project = ""; } \
               { \
                  if (index($0, ":") != 0) { \
                     project = $0; \
                     sub(":", "", project); \
                     } \
                  else \
                     if ($0 != "" && index($0, " item") == 0 ) \
                        printf("%s/%s\n", project, $0); \
               } ' | sed '/\/\$/d' \
                   | sed "s{$EZ_RELEASE_ROOT/$FAMILY_DIR/{\$$ROOT_VAR/{" \
                   | egrep -v "$FILE_EGREP_LIST" > $TMP_FILE
         elif [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
            svn list -R . | sed "s{^{\$$ROOT_VAR/{" | \
                egrep -v "$FILE_EGREP_LIST" > $TMP_FILE
         else
            echo "ERROR: Unsupported source control $SOURCE_CONTROL_SYSTEM."
         fi
      elif [ "$LIST_MODE" = "directories" ] ; then
         echo "\$$ROOT_VAR/../$FAMILY_DIR top" > $TMP_FILE
         find . -type d | egrep -v "$DIR_EGREP_LIST|^./$FAMILY_DIR\$" | \
                          sed "s{.{\$$ROOT_VAR{" | \
                          sed "s{\${ $FAMILY_DIR{" >> $TMP_FILE
      fi

      dos2unix.sh $TMP_FILE

      cd $THE_LIST_DIR
      src.ksh stabalize $STABALIZE_ANSWER -r -t Unchanged "" \
                    "$TMP_FILE" "$THE_LIST_FILE"
      NUMBER=`wc -l $THE_LIST_FILE | awk '{print $1}'`
      echo "Successfully found $NUMBER $PRETTY_NAME entries in $ROOT_DIR."
   done
}

cd $EZ_FAMILYTREE_ROOT
src.ksh test mk.sh
if [ $? -ne 0 ] ; then
   echo "ERROR: Unable to communicate with the Source Control server."
   exit 1
fi

for FILE_TYPE in $FILE_TYPE_LIST
do
   MakeFamilyList $FILE_TYPE
done
