#===========================================================================
#  Filename: CreatePatchStream.ksh
#    Author: Craig Schneider
# Orig Date: 04/21/09
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Create a patch stream.
#===========================================================================

. $EZ_SHARED_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: CreatePatchStream [-DryRun] [-Destroy | -Recreate] [-RelSameRev]
                         [-SkipCleanup] [-SkipCreate] [-SkipExtract]
                         [-SkipProfile] [-SkipRelUpdates] -Stream StreamName

       -Destroy: Destroy the patch stream.
        -DryRun: Display the commands to be executed, without executing them.
      -Recreate: Recreate the patch stream.
   -SkipCleanup: Skip the cleanup of the unused built and open files.
    -SkipCreate: Skip the source control creation portion of the release.
   -SkipExtract: Skip the extraction of the release.
   -SkipProfile: Skip the creation of the release profile file.
-SkipRelUpdates: Skip the creation of the release profile file.
        -Stream: Specify the name of the patch stream to be created.

!
   exit 1
}

EZ_DRYRUN=FALSE
DRYRUN_ARGS=
DESTROY_MODE=FALSE
typeset -u DO_CLEANUP=Y
typeset -u DO_CREATE=Y
typeset -u DO_EXTRACT
typeset -u DO_PROFILE=Y
typeset -u DO_REL_UPDATES=Y
RECREATE_MODE=FALSE
REL_SAME_REV=FALSE
STREAM_NAME=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
       -destroy) DESTROY_MODE=TRUE
                 shift ;;
        -dryrun) EZ_DRYRUN=TRUE
                 DRYRUN_ARGS="-DryRun"
                 shift ;;
      -recreate) RECREATE_MODE=TRUE
                 shift ;;
    -relsamerev) REL_SAME_REV=TRUE
                 shift ;;
   -skipcleanup) DO_CLEANUP=N
                 shift ;;
    -skipcreate) DO_CREATE=N
                 shift ;;
   -skipextract) DO_EXTRACT=N
                 shift ;;
   -skipprofile) DO_PROFILE=N
                 shift ;;
-skiprelupdates) DO_REL_UPDATES=N
                 shift ;;
        -stream) STREAM_NAME="$2"
                 shift
                 shift ;;
              *) if [ "$1" != "-?" ] ; then
                    Usage "Invalid argument: $1"
                 else
                    Usage
                 fi ;;
    esac
done

if [ "$STREAM_NAME" = "" ] ; then
   Usage "Must specify -stream StreamName"
fi

PATCH_STREAM_NUM_X="2.9.X"
PATCH_STREAM_NAME="${STREAM_NAME}${PATCH_STREAM_NUM_X}"

if [ "$DO_CREATE" = Y ] ; then
   # optionally destroy a pre-existing patch stream
   if [ "$DESTROY_MODE" = TRUE -o "$RECREATE_MODE" = TRUE ] ; then
      if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
         if [ "$EZ_DRYRUN" = FALSE ] ; then
             svn rm "$EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME" \
                 -m "Deleting patch stream for $PATCH_STREAM_NAME." 2>&1 | \
                 sed -e '/^$/d' \
                     -e '/Committed revision/d' \
                     -e '/does not exist/d'
            echo -n "Successfully destroyed "
            echo "$EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME."
         else
            echo -n "svn rm $EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME "
            echo    "-m 'Deleting patch stream for $PATCH_STREAM_NAME.'"
            echo -n "Would have destroyed "
            echo "$EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME."
         fi
      elif [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
         ss Destroy $/Releases/$PATCH_STREAM_NAME $EZ_VSS_OPTIONS -i-Y 2>&1 | \
                 sed '/^$/d' | \
               egrep -v "$THE_RELEASE|You are about to permanently remove|Continue anyway|irreversible change|changing to root project"
         echo "Successfully destroyed patch stream $/Releases/$PATCH_STREAM_NAME."
      else
         echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
         exit 1
      fi
      if [ "$DESTROY_MODE" = TRUE ] ; then
         exit 0
      fi
   fi

   OUTPUT_FILE="$TMPDIR/cps.$$.tmp"

   # Create the root project for the patch stream
   if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
      if [ "$EZ_DRYRUN" = FALSE ] ; then
         svn mkdir "$EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME" \
             -m "Created patch stream for $PATCH_STREAM_NAME." > "$OUTPUT_FILE"
         CREATE_EXIT_CODE=$?
         cat "$OUTPUT_FILE" | sed -e '/^$/d' \
                                  -e '/Committed revision/d'
         if [ $CREATE_EXIT_CODE -ne 0 ] ; then
            echo
            echo -n "ERROR: Failed creating "
            echo    "$EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME."
            grep "405 Method Not Allowed" "$OUTPUT_FILE"
            if [ $? -eq 1 ] ; then
               echo -n "ERROR: The release already exists, use the "
               echo    "-Recreate argument."
            fi
            exit 1
         fi
         echo -n "Successfully created "
         echo "$EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME."
      else
         echo -n "svn mkdir $EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME "
         echo    "-m 'Created patch stream for $PATCH_STREAM_NAME.'"
         echo -n "Would have created "
         echo "$EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME."
      fi
   elif [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
      ss Create $/Releases/$PATCH_STREAM_NAME \
                -C"Patch Stream For $PATCH_STREAM_NAME" \
                $EZ_VSS_OPTIONS > "$OUTPUT_FILE"
      CREATE_EXIT_CODE=$?
      cat "$OUTPUT_FILE" | grep -v $PATCH_STREAM_NAME
      if [ $CREATE_EXIT_CODE -ne 0 ] ; then
         echo "ERROR: Failed creating project $/Releases/$PATCH_STREAM_NAME."
         exit 1
      fi
      echo "Successfully created project $/Releases/$PATCH_STREAM_NAME."
   else
      echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
      exit 1
   fi
fi

function ShareRelease
{
   TARGET_ROOT="$1"
   shift
   SOURCE_ROOT="$1"
   shift
   SOURCE_FAMILY="$1"
   shift

   if  [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
      if [ "$EZ_DRYRUN" = FALSE ] ; then
         svn copy "$EZ_SOURCE_CONTROL_ROOT/$SOURCE_ROOT/$SOURCE_FAMILY" \
                  "$EZ_SOURCE_CONTROL_ROOT/$TARGET_ROOT/$SOURCE_FAMILY" \
                  -m "Copied for patch stream $PATCH_STREAM_NAME." | \
             sed -e '/^$/d' \
                 -e '/Committed revision/d'
         echo -n "Successfully copied "
         echo "$EZ_SOURCE_CONTROL_ROOT/$SOURCE_ROOT/$SOURCE_FAMILY."
      else
         echo -n "svn copy $EZ_SOURCE_CONTROL_ROOT/$SOURCE_ROOT/$SOURCE_FAMILY"
         echo -n " $EZ_SOURCE_CONTROL_ROOT/$TARGET_ROOT/$SOURCE_FAMILY"
         echo    " -m 'Copied for patch stream $PATCH_STREAM_NAME.'"
         echo -n "Would have copied "
         echo "$EZ_SOURCE_CONTROL_ROOT/$SOURCE_ROOT/$SOURCE_FAMILY."
      fi
   elif  [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
      ss CP $TARGET_ROOT $EZ_VSS_OPTIONS >$EZ_DEV_NULL 2>&1
      if [ $? -ne 0 ] ; then
         echo "ERROR: Failed setting project to $TARGET_ROOT."
         exit 1
      fi
   
      ss Share $SOURCE_ROOT/$SOURCE_FAMILY -R -C"Linked patch stream." \
               $EZ_VSS_OPTIONS > "$OUTPUT_FILE"
      SHARE_EXIT_CODE=$?
      cat "$OUTPUT_FILE" | grep -v $TARGET_ROOT
      if [ $SHARE_EXIT_CODE -ne 0 ] ; then
         echo "ERROR: Failed sharing $SOURCE_ROOT/$SOURCE_FAMILY."
         exit 1
      fi
      echo "Successfully shared $SOURCE_ROOT/$SOURCE_FAMILY."
   else
      echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
      exit 1
   fi
}

function ExtractRelease
{
   EXTRACT_PATCH_DIR=C:/$PATCH_STREAM_NAME
   if [ "$DO_EXTRACT" = "" ] ; then
      echo
      echo -n "Extract the release to $EXTRACT_PATCH_DIR <y,n>? "
      read DO_EXTRACT
   fi
   if [ "$DO_EXTRACT" = Y ] ; then
      if [ -d "$EXTRACT_PATCH_DIR" ] ; then
         rm -rf "$EXTRACT_PATCH_DIR.old"
         mv "$EXTRACT_PATCH_DIR" "$EXTRACT_PATCH_DIR.old"
      fi
      # NOTE: From C:/ need to create the PatchDir, do not
      #       do mkdir C:/PatchDir or perms will be wrong.
      cd "C:/"
      mkdir "$PATCH_STREAM_NAME"
      if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
         if [ "$EZ_DRYRUN" = FALSE ] ; then
             svn checkout "$EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME"
             if [ $? -eq 0 ] ; then
                echo "Successfully extracted to $EXTRACT_PATCH_DIR."
                rm -rf "$EXTRACT_PATCH_DIR.old"
             else
                echo "ERROR: Failed extracting to $EXTRACT_PATCH_DIR."
                exit 1
             fi
         else
             echo -n "svn checkout "
             echo    "$EZ_SOURCE_CONTROL_ROOT/Releases/$PATCH_STREAM_NAME."
             echo "Would have extracted to $EXTRACT_PATCH_DIR."
         fi
      else
         echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
         exit 1
      fi
   fi
}

function CreateProfile
{
   if [ "$DO_PROFILE" = N ] ; then
      return 0
   fi

   PROFILE_DIR="C:/PatchStreamHome"
   if [ ! -d "$PROFILE_DIR" ] ; then
      echo "ERROR: Profile directory $PROFILE_DIR does not exist."
      exit 1
   fi
   PROFILE_FILE_NAME="${PATCH_STREAM_NAME}_profile.ksh"
   PROFILE_FILE_FULL_NAME="$PROFILE_DIR/$PROFILE_FILE_NAME"
   if [ -f "$PROFILE_FILE_FULL_NAME" ] ; then
      echo "Successfully detected $PROFILE_FILE_FULL_NAME already exists."
   else
      cat > "$PROFILE_FILE_FULL_NAME" <<!
export PROFILE_UPDATES=FALSE
export EZ_ACTIVE_RELEASE=$PATCH_STREAM_NAME

if [ "\$1" != "" ] ; then
   /usr/bin/mksh -l -o vi -c "\$@"
else
   /usr/bin/mksh -l -o vi
fi
!
      PROFILE_FILE_FULL_NAME_KSH=`EZPath2Ksh $PROFILE_FILE_FULL_NAME`
      chmod 777 "$PROFILE_FILE_FULL_NAME_KSH"
      echo "Successfully created profile file $PROFILE_FILE_FULL_NAME."
   fi
}

function UpdateRelFiles
{
   if [ "$DO_REL_UPDATES" = N ] ; then
      return 0
   fi

   cd C:/
   PATCH_NUM_0="${PATCH_STREAM_NUM_X%X}0"
   PATCH_NUM_1="${PATCH_STREAM_NUM_X%X}1"
   if [ "$REL_SAME_REV" = TRUE ] ; then
      PATCH_NUM_1="$PATCH_NUM_0"
   fi
   for REL_FILE in `dlook.sh -d $PATCH_STREAM_NAME *.rel`
   do
#          -e "s/APPLICATION_VERSION=\"$PATCH_NUM_0\"/APPLICATION_VERSION=\"$PATCH_NUM_1\"/"
      sed -e "s/BRANCH_TYPE=MAIN/BRANCH_TYPE=PATCH/" \
          $REL_FILE > $REL_FILE.fix
      src.ksh stabalize -diffs -r $REL_FILE.fix $REL_FILE
   done

   TOP_DIR="${EZ_FAMILYTREE_ROOT%/*}"
   TOP_DIR_NAME="${TOP_DIR##*/}"
   PATCH_NUM_TEMP="${PATCH_STREAM_NUM_X%.X}"
   PATCH_NUM_MINOR="${PATCH_NUM_TEMP##*.}"
   let PATCH_NUM_MINOR=PATCH_NUM_MINOR+1
   PATCH_NUM_NEXT="${PATCH_NUM_TEMP%.*}.$PATCH_NUM_MINOR.0"
   if [ "$REL_SAME_REV" = TRUE ] ; then
      PATCH_NUM_NEXT="$PATCH_NUM_0"
   fi
   cd "$TOP_DIR/.."
   for REL_FILE in `dlook.sh -d $TOP_DIR_NAME *.rel`
   do
      sed -e "s/APPLICATION_VERSION=\"$PATCH_NUM_0\"/APPLICATION_VERSION=\"$PATCH_NUM_NEXT\"/" \
          $REL_FILE > $REL_FILE.fix
      src.ksh stabalize -diffs -r $REL_FILE.fix $REL_FILE
   done
}

function CleanupUnusedFiles
{
   if [ "$DO_CLEANUP" = N ] ; then
      return 0
   fi

   cd C:/
   for OPEN_FILE in `ls $PATCH_STREAM_NAME/FamilyTree/Releases/Open/*.ksh`
   do
      OPEN_FILE_NAME="${OPEN_FILE##*/}"
      if [ "$OPEN_FILE_NAME" != working.ksh -a \
           "$OPEN_FILE_NAME" != "${PATCH_STREAM_NAME}.ksh" ] ; then
         src.ksh destroy $DRYRUN_ARGS $OPEN_FILE
      fi
   done

   TOP_DIR="${EZ_FAMILYTREE_ROOT%/*}"
   PATCH_TOP_DIR="C:/$PATCH_STREAM_NAME"
   PATCH_MAJ_MIN="${PATCH_STREAM_NUM_X%.X}"
   for PATCH_BUILT_FILE in `ls \
                          $PATCH_STREAM_NAME/FamilyTree/Releases/Built/*.ksh`
   do
      PATCH_BUILT_FILE_NAME="${PATCH_BUILT_FILE##*/}"
      PATCH_BUILT_FILE_REL="${PATCH_BUILT_FILE_NAME%%_*}"
      if [ ! -d "$PATCH_TOP_DIR/$PATCH_BUILT_FILE_REL" ] ; then
         # This is for a family that was not extracted.
         src.ksh destroy $DRYRUN_ARGS $PATCH_BUILT_FILE
      else
         BUILT_FILE="$TOP_DIR/FamilyTree/Releases/Built/$PATCH_BUILT_FILE_NAME" 
         if [ -f "$BUILT_FILE" ] ; then
            egrep $EZ_EGREP_QUIET "LOCKED=TRUE" "$BUILT_FILE"
            if [ $? -eq 1 ] ; then
               # built file exists but is not locked
               # if the releases were not properly locked,
               # the built files will be destroyed.
               src.ksh destroy $DRYRUN_ARGS $PATCH_BUILT_FILE
            else
               BUILT_TEMP="${PATCH_BUILT_FILE_NAME#*_}"
               BUILT_MAJ="${BUILT_TEMP%%.*}"
               BUILT_TEMP="${BUILT_TEMP#$BUILT_MAJ.}"
               BUILT_MIN="${BUILT_TEMP%%.*}"
               BUILT_MAJ_MIN="$BUILT_MAJ.$BUILT_MIN"
               if [ "$PATCH_MAJ_MIN" != "" -a "$BUILT_MAJ_MIN" != "" -a \
                    "$PATCH_MAJ_MIN" != "$BUILT_MAJ_MIN" ] ; then
                  # exists and locked but major.minor numbers are different
                  src.ksh destroy $DRYRUN_ARGS $PATCH_BUILT_FILE
               #else
               #   exists and locked and major.minor numbers are the same
               fi
            fi
         else
            # already destroyed in working built heirarchy
            src.ksh destroy $DRYRUN_ARGS $PATCH_BUILT_FILE
         fi
      fi
   done
}

if [ "$DO_CREATE" = Y ] ; then
   # For each family, share over the desired version.
   if [ "$STREAM_NAME" = "CFour" ] ; then

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/TestCFour_2.9.0.8447 \
                   FamilyTree

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/CFour_2.9.0.8447 \
                   CFour

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/TestCFour_2.9.0.8447 \
                   TestCFour

   elif [ "$STREAM_NAME" = "Coins" ] ; then

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/Coins_2.6.0.7587 \
                   FamilyTree

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/Coins_2.6.0.7587 \
                   Coins

   elif [ "$STREAM_NAME" = "Conductor" ] ; then

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/TestConductor_2.9.0.2183 \
                   FamilyTree

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/Conductor_2.9.0.2183 \
                   NP

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/TestConductor_2.9.0.2183 \
                   TestNP

   elif [ "$STREAM_NAME" = "Charter_Impact" ] ; then

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/Charter_Impact_2.3.0.645 \
                   FamilyTree

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/Charter_Impact_2.3.0.645 \
                   Charter_Impact

   elif [ "$STREAM_NAME" = "TWC_NetMon" ] ; then

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/TWC_NetMon_2.3.0.649 \
                   FamilyTree

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/TWC_NetMon_2.3.0.649 \
                   TWC_NetMon

   elif [ "$STREAM_NAME" = "TWC_PerfDash" ] ; then

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/TWC_PerfDash_2.3.0.621 \
                   FamilyTree

      ShareRelease Releases/$PATCH_STREAM_NAME \
                   Releases/TWC_PerfDash_2.3.0.621 \
                   TWC_PerfDash

   else
      echo "ERROR: Unsupported stream name $STREAM_NAME."
      exit 1
   fi
fi

ExtractRelease

CreateProfile

UpdateRelFiles

CleanupUnusedFiles

rm -f "$OUTPUT_FILE"
exit 0
