#===========================================================================
#  Filename: rel_archive.ksh
#    Author: Craig Schneider
# Orig Date: 05/31/02
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Archive/unarchive releases into another storage area.
#===========================================================================

. $EZ_SHARED_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$1"
   fi
   echo
   echo "Usage: archive {-put|-get} [release_name]"
   echo 
   exit 1
}

#===========================================================================
# The Main.
#===========================================================================
ARCH_REL_NAME=""

# make sure there are one or two args only
if [ "$#" -gt 2 -o "$#" -lt 1 ]; then
    usage "Incorrect number of arguments."
fi

TYPE=""
while [ "$#" -ne 0 ]
do
   case $1 in
     -put|-p)
        TYPE=PUT
        shift
        ;;
     -get|-g)
        TYPE=GET
        shift
        ;;
     -?) usage
        ;;
     *)
        if [ "$#" -gt 0 -a "$TYPE" = "" ]; then
           usage
        fi
        ARCH_REL_NAME=$1
        shift
        ;;
    esac
done

RELEASE_INFO_DIR=$EZ_FAMILYTREE_ROOT/Releases/Built
if [ ! -d "$RELEASE_INFO_DIR" ] ; then
   echo "ERROR: Release info directory $RELEASE_INFO_DIR not found!"
   exit 1
fi

if [ "${EZ_FAMILYTREE_ROOT#$EZ_BUILD_ROOT}" != "$EZ_FAMILYTREE_ROOT" ] ; then
   echo
   echo -n "Detected attempt to $0 from $EZ_ACTIVE_RELEASE, continue <y,n>? "
   read ANSWER
   if [ "$ANSWER" != Y -a "$ANSWER" != y ] ; then
      exit 1
   fi
fi

#
# determine the release to archive/unarchive
#
rel_pick -a $ARCH_REL_NAME
if [ "$THE_RELEASE" = "" ] ; then
   echo "ERROR: Failed To Select A Release, Quitting!"
   exit 1
fi

RELEASE_INFO_FILE=$THE_RELEASE.ksh

cd $RELEASE_INFO_DIR
if [ ! -f $RELEASE_INFO_FILE ] ; then
   src extract $RELEASE_INFO_FILE
fi

TMP_REL_INFO="$TMPDIR/rel_info.$$"
egrep "ARCHIVED=|LOCKED=|DELIVER_LIST=" $RELEASE_INFO_FILE > "$TMP_REL_INFO"
. "$TMP_REL_INFO"

if [ "$ARCHIVED" = "TRUE" -a "$TYPE" = "PUT" ]; then
    echo
    echo "ERROR: Release $THE_RELEASE is already archived."
    exit 1
elif [ "$ARCHIVED" != "TRUE" -a "$TYPE" = "GET" ]; then
    echo
    echo "ERROR: Release $THE_RELEASE is not currently archived."
    exit 1
fi

if [ "$LOCKED" != "TRUE" -a "$TYPE" = "PUT" ]; then
    echo
    echo "Warning: Release $THE_RELEASE is NOT LOCKED but will be archived anyway."
fi

function ArchiveDirectory
{
   DIR_TO_ARCHIVE="$1"

   # begin the actual archive
   if [ ! -d $EZ_ARCHIVE_ROOT/$THE_RELEASE ]; then
      mkdir -p $EZ_ARCHIVE_ROOT/$THE_RELEASE
   fi

   # Need a prefix because build and release dirs can have the same name.
   ARCHIVE_PREFIX="rdir_"
   if [ "${DIR_TO_ARCHIVE#$EZ_BUILD_ROOT}" != "$DIR_TO_ARCHIVE" ] ; then
      ARCHIVE_PREFIX="bdir_"
   fi
   DIR_TO_ARCHIVE_BASE=${DIR_TO_ARCHIVE##*/}
   DIR_TO_ARCHIVE_PARENT=${DIR_TO_ARCHIVE%/*}
   ARCHIVE_FILE_NAME=$ARCHIVE_PREFIX$DIR_TO_ARCHIVE_BASE.gtz
   ARCHIVE_FILE_PATH=$EZ_ARCHIVE_ROOT/$THE_RELEASE/$ARCHIVE_FILE_NAME

   if [ "$TYPE" = GET ] ; then

      ##################### Restore Here
      if [ ! -d $DIR_TO_ARCHIVE ] ; then
         echo "ERROR: The restore directory $DIR_TO_ARCHIVE does not exist."
         exit 1
      fi
      if [ ! -f $DIR_TO_ARCHIVE/ARCHIVED ] ; then
         echo "ERROR: The restore directory $DIR_TO_ARCHIVE was not archived."
         exit 1
      fi
      if [ ! -f $ARCHIVE_FILE_PATH ] ; then
         echo "ERROR: The archived file $ARCHIVE_FILE_PATH does not exist."
         exit 1
      fi
      cd $DIR_TO_ARCHIVE_PARENT
      gtar -xzf $ARCHIVE_FILE_PATH > /dev/null 2>&1
      if [ $? -eq 0 ] ; then
         echo "Successfully restored $DIR_TO_ARCHIVE."
      else
         echo "ERROR: Failed to restore $DIR_TO_ARCHIVE."
         exit 1
      fi
      if [ "$LOCKED" = TRUE ] ; then
         chmod -R 555 $DIR_TO_ARCHIVE
      fi
      rm -f $DIR_TO_ARCHIVE/ARCHIVED
      rm -f $ARCHIVE_FILE_PATH

   else

      ##################### Archive Here
      if [ -f $ARCHIVE_FILE_PATH ] ; then
         echo "Warning: The archive file $ARCHIVE_FILE_PATH already exists."
         return
      fi

      cd $DIR_TO_ARCHIVE_PARENT
      gtar -czf $ARCHIVE_FILE_PATH $DIR_TO_ARCHIVE_BASE > /dev/null 2>&1
      if [ $? -eq 0 ] ; then
         echo "Successfully archived $DIR_TO_ARCHIVE."
      else
         echo "ERROR: Failed to archive $DIR_TO_ARCHIVE."
         exit 1
      fi

      RUN_WHOAMI=`whence whoami`
      CURRENT_USER=`$RUN_WHOAMI`
      /usr/local/bin/take_ownership.ksh $DIR_TO_ARCHIVE $CURRENT_USER rel_lock.ksh
      chmod -R +w $DIR_TO_ARCHIVE
      rm -rf $DIR_TO_ARCHIVE/*
      echo "Directory archived on: `date '+%m/%d/%y'`" >> $DIR_TO_ARCHIVE/ARCHIVED
   fi 
}


COMPACT_VERSION=${THE_RELEASE##*_}
for DELIVER_FILE in $DELIVER_LIST
do
   DELIVER_FILE_NAME=${DELIVER_FILE##*/}
   DELIVER_EXTENSION=${DELIVER_FILE##*.}
   DELIVER_DIR_NAME=${DELIVER_FILE_NAME%.deliver}
   DELIVER_DIR_NAME=${DELIVER_DIR_NAME}_$COMPACT_VERSION
   ARCHIVE_DIR_NAME=$EZ_ARCHIVE_ROOT/$DELIVER_DIR_NAME
   RELEASE_DIR_NAME=$EZ_DELIVER_ROOT/$DELIVER_DIR_NAME

   ArchiveDirectory "$RELEASE_DIR_NAME"
done

ArchiveDirectory "$EZ_BUILD_ROOT/$THE_RELEASE"

if [ "${TYPE}" = "GET" ]; then
   rmdir $EZ_ARCHIVE_ROOT/$THE_RELEASE
fi

function UpdateLocalReleaseInfoFile
{
   TMP_FILE="$TMPDIR/rel_archive.$$"
   ALREADY_HAVE_FILE=FALSE

   if [ "$ARCHIVED" != "" ]; then
       # file has a random archived in there or is being restored
       src get $RELEASE_INFO_FILE
       ALREADY_HAVE_FILE=TRUE
       sed '/^ARCHIVED/d' $RELEASE_INFO_FILE > "$TMP_FILE"
       mv "$TMP_FILE" "$RELEASE_INFO_FILE"
   fi

   if [ "$TYPE" = "PUT" ]; then
       if [ -f $RELEASE_INFO_FILE ]; then
           # check to see if file already checked out
           if [ "$ALREADY_HAVE_FILE" = "FALSE" ]; then
               src get $RELEASE_INFO_FILE
           fi
           sed '/RELEASED_LIST/a\
ARCHIVED=TRUE' $RELEASE_INFO_FILE > "$TMP_FILE"
           mv "$TMP_FILE" "$RELEASE_INFO_FILE"
           src put -m "Archiving the release" $RELEASE_INFO_FILE
       else
           echo "ERROR: Failed finding release file to update status."
           exit 1
       fi
   else
       src put -m "UnArchiving the release" $RELEASE_INFO_FILE
   fi
}

# Set the release file to ARCHIVED or NOT
cd $RELEASE_INFO_DIR
UpdateLocalReleaseInfoFile
echo "Successfully updated $THE_RELEASE.ksh."

# Set the working release file to ARCHIVED or NOT
EZ_REL_DIR="${EZ_ENV_DIR_ORIG%/*}/Releases/Built"
if [ "$RELEASE_INFO_DIR" != "$EZ_REL_DIR" -a \
     -f $EZ_REL_DIR/$THE_RELEASE.ksh ] ; then
   PREV_PWD=$PWD
   cd $EZ_REL_DIR
   UpdateLocalReleaseInfoFile
   cd $PREV_PWD
   echo "Successfully updated working version of $THE_RELEASE.ksh."
fi


if [ "$TYPE" = "PUT" ]; then
   echo "Successfully archived the release $THE_RELEASE"
else
   echo "Successfully restored the release $THE_RELEASE"
fi

exit 0

