#===========================================================================
#  Filename: rel_destroy.ksh
#    Author: Craig Schneider
# Orig Date: 03/21/97
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Remove all traces of a candidate release.
#            This is usually done when the release has been superseded.
#===========================================================================

. $EZ_SHARED_ENV

if [ "$1" = -? ] ; then
   echo
   echo "Usage: destroy [RELEASE_NAME]"
   echo
   exit 1
fi

function PerformRelDestroy
{
   DESTROY_REL_NAME=$1

   RELEASE_INFO_DIR=$EZ_FAMILYTREE_ROOT/Releases/Built
   if [ ! -d "$RELEASE_INFO_DIR" ] ; then
      echo "ERROR: Release info directory $RELEASE_INFO_DIR not found!"
      exit 1
   fi

   if [ "${EZ_FAMILYTREE_ROOT#$EZ_BUILD_ROOT}" != "$EZ_FAMILYTREE_ROOT" ] ; then
      echo
      echo -n "Detected attempt to $0 from $EZ_ACTIVE_RELEASE, continue <y,n>? "
      read ANSWER
      if [ "$ANSWER" != Y -a "$ANSWER" != y ] ; then
         exit 1
      fi
   fi

   rel_pick -a $DESTROY_REL_NAME
   if [ "$THE_RELEASE" = "" ] ; then
      echo "Failed To Select A Release, Quitting!"
      exit 1
   fi

   cd $RELEASE_INFO_DIR
   RELEASE_INFO_FILE=$THE_RELEASE.ksh
   echo
   echo

   # check to see if the release has any references
   rel.sh select -list -for_destroy -v $THE_RELEASE
   REL_SELECT_EXIT_CODE=$?
   if [ $REL_SELECT_EXIT_CODE -eq 2 ] ; then
      echo "ERROR: Detected attempt to delete $THE_RELEASE which is locked."
      exit 1
   elif [ $REL_SELECT_EXIT_CODE -ne 0 ] ; then
      echo -n "Warning: Release ${THE_RELEASE%%_*} "
      echo -n "has references, continue anyway <y,n>? "
      read ANSWER
      if [ "$ANSWER" != "y" -a "$ANSWER" != "Y" ] ; then
         exit 1
      fi
   fi

   # check to see if the release is locked
   grep "^LOCKED=TRUE" $RELEASE_INFO_FILE > $TMPDIR/rel_destroy.tmp
   if [ $? -eq 0 ]; then
      echo "ERROR: Detected attempt to delete $THE_RELEASE which is locked."
      exit 1
   fi

   if [ ! -f $RELEASE_INFO_FILE ] ; then
      src.ksh extract $RELEASE_INFO_FILE
   fi
   eval `egrep "^BRANCH_TYPE=|^SOURCED_LIST=|^RELEASED_LIST=|^DELIVER_LIST=|^DELIVER_TARGET_SYS_DIR=|^DELIVER_TARGET_SYS_PREFIX=|^DESTROY_DIRECTORIES=|^TURNOVER_DELIVER_LIST=" \
         $RELEASE_INFO_FILE`

   BUILD_VERSION=${THE_RELEASE##*_}
   for DELIVER_FILE in $DELIVER_LIST $TURNOVER_DELIVER_LIST
   do
      DELIVER_FILE_NAME=${DELIVER_FILE##*/}
      DELIVER_EXTENSION=${DELIVER_FILE##*.}
      DELIVER_DIR_NAME=${DELIVER_FILE_NAME}
      DELIVER_DIR_NAME=${DELIVER_DIR_NAME%.ksh}       # want .ksh first to
      DELIVER_DIR_NAME=${DELIVER_DIR_NAME%.deliver}   # support .deliver.ksh 
      DELIVER_DIR_NAME=${DELIVER_DIR_NAME}_$BUILD_VERSION
      ARCHIVE_DIR_NAME=$EZ_ARCHIVE_ROOT/$DELIVER_DIR_NAME
      DELIVER_TARGET_ROOT="$EZ_DELIVER_ROOT"
      if [ "$DELIVER_TARGET_SYS_DIR" != "" ] ; then
         DELIVER_TARGET_ROOT="$DELIVER_TARGET_ROOT/$DELIVER_TARGET_SYS_DIR"
      fi
      if [ "$DELIVER_TARGET_SYS_PREFIX" != "" ] ; then
         DELIVER_TARGET_ROOT="$DELIVER_TARGET_ROOT/$DELIVER_TARGET_SYS_PREFIX"
         DELIVER_TARGET_ROOT="$DELIVER_TARGET_ROOT$BUILD_VERSION"
      fi
      RELEASE_DIR_NAME=$DELIVER_TARGET_ROOT/$DELIVER_DIR_NAME

      #echo "DELIVER_TARGET_ROOT=$DELIVER_TARGET_ROOT"
      #echo "DELIVER_FILE_NAME=$DELIVER_FILE_NAME"
      #echo "RELEASE_DIR_NAME=$RELEASE_DIR_NAME"

      if [ -d $RELEASE_DIR_NAME ] ; then
         # CYGWIN 1.7 started complaining about Permission denied
         #            what the hell happened to the -f Force argument???
         if [ "$EZ_KSH_BASE" = CYGWIN ] ; then
            rm -rf $RELEASE_DIR_NAME 2>&1 | \
                              egrep -v "Directory not empty|Permission denied"
            if [ -d $RELEASE_DIR_NAME ] ; then
               chmod --silent --recursive 777 $RELEASE_DIR_NAME
            fi
         fi
         rm -rf $RELEASE_DIR_NAME
         if [ $? -ne 0 ] ; then
            echo ERROR: Failed destroying release directory $RELEASE_DIR_NAME.
            exit 1
         fi
         echo "Successfully destroyed release directory $RELEASE_DIR_NAME."
      fi

      if [ "$DELIVER_TARGET_SYS_PREFIX" != "" -a \
           -d "$DELIVER_TARGET_ROOT" ] ; then
         rmdir --ignore-fail-on-non-empty "$DELIVER_TARGET_ROOT"
      fi

      if [ -d $ARCHIVE_DIR_NAME ] ; then
         rm -rf $ARCHIVE_DIR_NAME
         if [ $? -ne 0 ] ; then
            echo ERROR: Failed destroying archive directory $ARCHIVE_DIR_NAME.
            exit 1
         fi
         echo "Successfully destroyed archive directory $ARCHIVE_DIR_NAME."
      fi
   done

   ARCHIVE_DIR=$EZ_ARCHIVE_ROOT/$THE_RELEASE
   if [ -d $ARCHIVE_DIR ] ; then
      rm -rf $ARCHIVE_DIR
      if [ $? -ne 0 ] ; then
         echo "ERROR: Failed destroying archive directory $ARCHIVE_DIR."
         exit 1
      fi
      echo "Successfully destroyed archive directory $ARCHIVE_DIR."
   fi

   BUILD_DIR=$EZ_BUILD_ROOT/$THE_RELEASE
   # CYGWIN 1.7 started complaining about Permission denied
   #            what the hell happened to the -f Force argument???
   if [ -d $BUILD_DIR ] ; then
      if [ "$EZ_KSH_BASE" = CYGWIN ] ; then
         rm -rf $BUILD_DIR 2>&1 | egrep -v "Directory not empty|Permission denied"
         if [ -d $BUILD_DIR ] ; then
            chmod --silent --recursive 777 $BUILD_DIR
         fi
      fi
      if [ -d $BUILD_DIR ] ; then
         rm -rf $BUILD_DIR
         if [ $? -ne 0 ] ; then
            echo "ERROR: Failed destroying build directory $BUILD_DIR."
            exit 1
         fi
      fi
      echo "Successfully destroyed build directory $BUILD_DIR."
   fi

   BUILD_ZIP_FILE=$BUILD_DIR.zip
   if [ -f $BUILD_ZIP_FILE ] ; then
      rm -f $BUILD_ZIP_FILE
      if [ $? -ne 0 ] ; then
         echo "ERROR: Failed destroying build zip file $BUILD_ZIP_FILE."
         exit 1
      fi
      echo "Successfully destroyed build zip file $BUILD_ZIP_FILE."
   fi

   if [ "$DESTROY_DIRECTORIES" != "" ] ; then
      for DESTROY_DIR in $DESTROY_DIRECTORIES
      do
         rm -rf $DESTROY_DIR
         if [ $? -ne 0 ] ; then
            echo "ERROR: Failed destroying directory $DESTROY_DIR."
            exit 1
         fi
         echo "Successfully destroyed directory $DESTROY_DIR"
      done
   fi

   if [ "$EZ_SYSTEM_GROUP" = "PCC" ] ; then
      rel.sh 3rd_ref -Delete -r $THE_RELEASE
      if [ $? -ne 0 ] ; then
         echo "ERROR: Failed removing 3rd party references."
         exit 1
      fi
   fi

   if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
      svn delete -m "Destroyed release $THE_RELEASE." \
          $EZ_SOURCE_CONTROL_ROOT/Releases/$THE_RELEASE 2>&1 | \
          sed '/^$/d' | egrep -v "^Committ|does not exist"
   elif [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
      ss CP \$/Releases/$THE_RELEASE $EZ_VSS_OPTIONS >$EZ_DEV_NULL 2>&1
      if [ $? -eq 0 ] ; then
         ss Destroy \$/Releases/$THE_RELEASE $EZ_VSS_OPTIONS -i-Y 2>&1 | \
            sed '/^$/d' | \
            egrep -v "$THE_RELEASE|You are about to permanently remove|Continue anyway|irreversible change|changing to root project"
         echo -n "Successfully destroyed source control "
         echo    "branches \$/Releases/$THE_RELEASE"
      fi
   else
      echo "ERROR: Unsupported Source Control System $SOURCE_CONTROL_SYSTEM."
      exit 1
   fi

   EZ_REL_DIR="${EZ_ENV_DIR_ORIG%/*}/Releases/Built"
   if [ -f $EZ_REL_DIR/$THE_RELEASE.ksh ] ; then
      PREV_PWD=$PWD
      cd $EZ_REL_DIR
      src.ksh destroy -force $THE_RELEASE.ksh
      echo "Successfully destroyed working version of $THE_RELEASE.ksh."
      cd $PREV_PWD
   fi

}

if [ $# -eq 0 ] ; then
   PerformRelDestroy
else
   for SINGLE_REL_NAME in $*
   do
      PerformRelDestroy $SINGLE_REL_NAME
   done
fi

exit 0
