#===========================================================================
#  Filename: src_get.ksh
#    Author: Craig Schneider
# Orig Date: 08/04/04
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Get files and/or directories from source control for editing.
#===========================================================================

. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src get [-d] [-u] [-m Message] ${PROMPT_OPT}FileName(s)...
               [-dryrun] [-quiet | -verbose]

          -d: Delete the file(s) from source control, calls src delete.
     -dryrun: Display the commands to be executed, without executing them.
 FileName(s): One or more file names to check out of source control.
          -m: The message to be used as the reason for checkout.
      -quiet: Suppress non critical messages about operations being performed.
          -u: Unget the files from source control, calls src unget.
    -verbose: Display verbose messages about operations being performed.

!
   exit 1
}

for ARGUMENT in "$@"
do
   # src get -u is an alias for src unget (which ignores the -u).
   if [ "$ARGUMENT" = "-u" ] ; then
      #echo "Warning: 'src get -u' has been deprecated use 'src unget'."
      src.ksh unget "$@"
      exit $?
   fi
   # src get -d is an alias for src delete (which ignores the -d).
   if [ "$ARGUMENT" = "-d" ] ; then
      #echo "Warning: 'src get -d' has been deprecated use 'src delete'."
      src.ksh delete "$@"
      exit $?
   fi
done

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi
if [ $# -eq 0 ] ; then
   Usage
fi

integer SRC_FILE_LIST_COUNT=0
EXIT_CODE=0
MESSAGE_TEXT="None"

EZ_DRYRUN=FALSE
EZ_VERBOSITY=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -d) Usage "ERROR: 'src get -d' has been deprecated use 'src delete'."
          shift ;;    # deprecated on 08/17/04
      -u) Usage "ERROR: 'src get -u' has been deprecated use 'src unget'."
          shift ;;    # deprecated on 08/17/04
      -x) Usage "ERROR: 'src get -x' has been deprecated use 'src extract'."
          shift ;;    # deprecated on 08/17/04
      -z) Usage "ERROR: 'src get -z' has been deprecated use 'src destroy'."
          shift ;;    # deprecated on 08/17/04
      -dryrun)
          EZ_DRYRUN=TRUE
          shift ;;
      -m) MESSAGE_TEXT="$2"
          shift
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) SRC_FILE_NAME_LIST[SRC_FILE_LIST_COUNT]="$1"
          SRC_FILE_LIST_COUNT=SRC_FILE_LIST_COUNT+1
          shift ;;
    esac
done

EZSrcSetCheckoutActivity

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
integer COUNT=0
while [ $COUNT -lt $SRC_FILE_LIST_COUNT ]
do
   SRC_FILE_NAME="${SRC_FILE_NAME_LIST[COUNT]}"
   COUNT=COUNT+1
   if [ -w "$SRC_FILE_NAME" -a  ! -d "$SRC_FILE_NAME" ] ; then
      echo -n "File $SRC_FILE_NAME is writable, checkout anyway <y,n>? "
      read ANSWER
      if [ "$ANSWER" != Y -a "$ANSWER" != y ] ; then
         echo "Skipping checkout of writable $(EZSrcObjectText $SRC_FILE_NAME)."
         continue
      fi
      if [ "$SOURCE_CONTROL_SYSTEM" = svn -o \
           "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
         EZInvokeCommand chmod 555 "$SRC_FILE_NAME"
      fi
   fi

   EZSrcMappedFileNameSetup "$SRC_FILE_NAME" LOCAL
   if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
      EZInvokeCommand -egrep_v "' locked by user '" \
           $SRC_GET_COMMAND \
           $SRC_MESSAGE_FLAG"$MESSAGE_TEXT"  "$SRC_MAPPED_FILE_NAME" 
      if [ $EZInvokeCommandExitCode -eq 0 ] ; then
         if [ "$EZ_VERBOSITY" != quiet ] ; then
            echo "Successfully CHECKED OUT $(EZSrcObjectText $SRC_FILE_NAME)."
         fi
      else
         echo "ERROR: Failed checking out $(EZSrcObjectText $SRC_FILE_NAME)."
         EXIT_CODE=1
      fi
   elif [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
      EZInvokeCommand -out_err_file $SRC_OUTPUT $SRC_GET_COMMAND \
           $SRC_MESSAGE_FLAG"$MESSAGE_TEXT"  "$SRC_MAPPED_FILE_NAME"
      if [ $? -eq 0 ] ; then
         if [ "$EZ_VERBOSITY" != quiet ] ; then
            echo "Successfully CHECKED OUT $(EZSrcObjectText $SRC_FILE_NAME)."
         fi
      else
         cat $SRC_OUTPUT
         echo "ERROR: Failed checking out $(EZSrcObjectText $SRC_FILE_NAME)."
         EXIT_CODE=1
      fi
   else
      echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
      EXIT_CODE=1
   fi
   EZSrcMappedDirNameRestore
done

EZSrcCleanup
exit $EXIT_CODE
