#===========================================================================
#  Filename: src_history.ksh
#    Author: Craig Schneider
# Orig Date: 02/20/13
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Get the history of changes for a source control file.
#===========================================================================

. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src history FileName(s)... [-ChangeCount] [-DryRun] [-LastChangeRev]
              [-Quiet | -Verbose]

  -ChangeCount: Display the change count for the file.
       -DryRun: Display the commands to be executed, without executing them.
   FileName(s): One or more file names to extract from source control.
-LastChangeRev: Display the last change revision for the file.
        -Quiet: Suppress non critical messages about operations being performed.
      -Verbose: Display verbose messages about operations being performed.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi
if [ $# -eq 0 ] ; then
   Usage
fi

integer SRC_FILE_LIST_COUNT=0
EXIT_CODE=0

EZ_DRYRUN=FALSE
EZ_VERBOSITY=
HISTORY_MODE="DEFAULT"

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -changecount)
          HISTORY_MODE="CHANGE_COUNT"
          shift ;;
      -dryrun)
          EZ_DRYRUN=TRUE
          shift ;;
      -lastchangerev)
          HISTORY_MODE="LAST_CHANGE_REV"
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) SRC_FILE_NAME_LIST[SRC_FILE_LIST_COUNT]="$1"
          SRC_FILE_LIST_COUNT=SRC_FILE_LIST_COUNT+1
          shift ;;
    esac
done

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
integer COUNT=0
while [ $COUNT -lt $SRC_FILE_LIST_COUNT ]
do
   SRC_FILE_NAME="${SRC_FILE_NAME_LIST[COUNT]}"
   COUNT=COUNT+1

   EZSrcMappedFileNameSetup "$SRC_FILE_NAME" LOCAL
   if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
      if [ "$HISTORY_MODE" = "CHANGE_COUNT" ] ; then
         REVISION_COUNT=`svn log -q $SRC_MAPPED_FILE_NAME 2> $EZ_DEV_NULL | \
             sed '/----------------------/d' | wc -l | awk '{print $1}'`
         echo "$REVISION_COUNT $SRC_FILE_NAME"
      elif [ "$HISTORY_MODE" = "LAST_CHANGE_REV" ] ; then
         LAST_CHANGE_REV=`svn info $SRC_MAPPED_FILE_NAME 2> $EZ_DEV_NULL | \
                         grep "Last Changed Rev:" | \
                         sed 's/Last Changed Rev: //'`
         if [ "$LAST_CHANGE_REV" = "" ] ; then
            LAST_CHANGE_REV="-1"
         fi
         if [ "$EZ_VERBOSITY" = "-verbose"  -o \
              $SRC_FILE_LIST_COUNT -gt 1 ] ; then
            echo "$LAST_CHANGE_REV $SRC_FILE_NAME"
         else
            echo "$LAST_CHANGE_REV"
         fi
      else
         CURR_REV=`EZSrcGetRepositoryRevision`
         if [ "$CURR_REV" = "" ] ; then
            echo "ERROR: Failed to determine repository revision."
            exit 1
         fi

         while [ 1 -eq 1 ]
         do
            EZInvokeCommand -out_err_file $SRC_OUTPUT $SRC_HISTORY_COMMAND \
                            -r $CURR_REV "$SRC_MAPPED_FILE_NAME"
            LAST_CHANGE_AUTHOR=`grep 'Last Changed Author:' $SRC_OUTPUT | \
                                cut -f2 -d: | awk '{ print $1 }'`
            LAST_CHANGE_REV=`grep 'Last Changed Rev:' $SRC_OUTPUT | \
                                cut -f2 -d: | awk '{ print $1 }'`
            LAST_CHANGE_DATE=`grep 'Last Changed Date:' $SRC_OUTPUT | \
                                cut -f2- -d: | awk '{ print $1 " " $2 }'`
            echo -n "$SRC_FILE_NAME $LAST_CHANGE_REV $LAST_CHANGE_AUTHOR \c"
            echo "$LAST_CHANGE_DATE"
            let CURR_REV=LAST_CHANGE_REV-1
            if [ $CURR_REV -lt 1 ] ; then
               break
            fi
         done
      fi
   else
      echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
      EXIT_CODE=1
   fi
   EZSrcMappedDirNameRestore
done

EZSrcCleanup
exit $EXIT_CODE
