#===========================================================================
#  Filename: src_out.ksh
#    Author: Craig Schneider
# Orig Date: 08/27/04
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: List files and/or directories checked out of source control.
#===========================================================================

. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi

   CVIEW_OPT=
   CVIEW_TEXT=
   UNV_OPT=
   UNV_TEXT=
   if [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
      UNV_OPT=" | -universe"
      UNV_TEXT="   -universe: List checked out files in the entire source control universe.@"
   fi

   (cat | tr '@' '\n') <<!

Usage: src out [-a | -u all | -u UserName(s) | -put | -unget]
               [-e | -f all | -f FamilyName(s)${UNV_OPT}]
               ${CVIEW_OPT}[-d] [-datetime] [-no_heading] [-show_user]
               [-dryrun] [-quiet | -verbose]

          -a: List files checked out by anyone, equivalent to '-u all'.
${CVIEW_TEXT}          -d: Limits list to items in the current directory only.
   -datetime: Display the date and time of the checkout.
     -dryrun: Display the commands to be executed, without executing them.
          -e: List files checked out everywhere, equivalent to '-f all'.  
          -f: Comma separated list of family names or the word 'all'.
 -no_heading: Disable display of the heading.
        -put: List checked out files formatted as put commands.
  -show_user: Display the name of the user who performed the checkout.
      -quiet: Suppress non critical messages about operations being performed.
          -u: Comma separated list of user names or the word 'all'.
      -unget: List checked out files formatted as unget commands.
${UNV_TEXT}    -verbose: Display verbose messages about operations being performed.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi

EXIT_CODE=0

EZ_DRYRUN=FALSE
EZ_VERBOSITY=

AS_PUT_COMMANDS=FALSE
AS_UNGET_COMMANDS=FALSE
SHOW_HEADING=TRUE
SHOW_DATE_TIME=FALSE
FORCE_SHOW_USER=FALSE
LIMIT_TO_CURRENT_DIR=FALSE
FAMILY_LIST=`EZSrcGetCurrentFamilyName`
SRC_STATUS_ARGS="-v"
USER_LIST="$LOGNAME"
if [ "$EZ_SRC_OUT_USER_LIST" != "" ] ; then
   USER_LIST="$EZ_SRC_OUT_USER_LIST"
fi
VIEW_SPECIFIER=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -a|-anyone) USER_LIST="anyone"
          shift ;;
      -cview) VIEW_SPECIFIER="$1"
          shift ;;
      -d) LIMIT_TO_CURRENT_DIR=TRUE
          shift ;;
      -datetime) SHOW_DATE_TIME=TRUE
          shift ;;
      -dryrun)
          EZ_DRYRUN=TRUE
          SRC_STATUS_ARGS="$SRC_STATUS_ARGS -dryrun"
          shift ;;
      -e|-everywhere) FAMILY_LIST="everywhere"
          shift ;;
      -f) FAMILY_LIST="$2"
          if [ "$FAMILY_LIST" = all ] ; then
             FAMILY_LIST=everywhere
          fi
          shift
          shift ;;
      -no_heading)
          SHOW_HEADING=FALSE
          shift ;;
      -put) AS_PUT_COMMANDS=TRUE
          shift ;;
      -show_user) FORCE_SHOW_USER=TRUE
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          SRC_STATUS_ARGS="$SRC_STATUS_ARGS $1"
          shift ;;
      -u) USER_LIST="$2"
          if [ "$USER_LIST" = all ] ; then
             USER_LIST=anyone
          fi
          shift
          shift ;;
      -unget) AS_UNGET_COMMANDS=TRUE
          shift ;;
      -universe) FAMILY_LIST="universe"
          shift ;;
      *)  if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
    esac
done

if [ "$FAMILY_LIST" = "" ] ; then
   Usage "ERROR: src out must be run with -f or executed under a valid family."
fi

#===========================================================================
# List the files checked out by the specified user in the specified family.
#===========================================================================
function EZSrcDisplayCheckedOutHeading
{
   FAMILY_NAME="$1"
   USER_LIST="$2"

   if [ "$SHOW_HEADING" = FALSE ] ; then
      return
   fi
   DISPLAY_USER_LIST="$USER_LIST"
   if [ "$USER_LIST" = anyone ] ; then
      DISPLAY_USER_LIST="all users"
   fi
   echo
   echo -n "Listing files checked out by $DISPLAY_USER_LIST "
   if [ "$FAMILY_NAME" = universe ] ; then
      echo "in the entire source control universe."
   else
      echo "under $FAMILY_NAME."
   fi
   echo -n "======================================"
   echo    "======================================"
}

#===========================================================================
# List the files checked out by the specified user in the specified family.
#===========================================================================
function EZSrcListCheckedOutVSS
{
   FAMILY_NAME="$1"
   USER_NAME="$2"
   SHOW_USER_NAME="$3"

   USER_NAME_OPTIONS="-U$USER_NAME"
   if [ "$USER_NAME" = "anyone" ] ; then
      USER_NAME_OPTIONS=""
   fi

   SRC_OUT_FILE="$TMPDIR/src_out.$$"
   
   MAPPED_ROOT=`EZSrcGetMappedFamilyRoot`
   if [ "$FAMILY_NAME" = universe ] ; then
      MAPPED_ROOT="\$/"
   fi
   SRC_OUT_FILE="$(Path2Win.ksh $SRC_OUT_FILE)"
   EZInvokeCommand $SRC_OUT_COMMAND $MAPPED_ROOT -R $USER_NAME_OPTIONS \
           "-O@$SRC_OUT_FILE"
   # Exit status of 0 means no checked out files found, 1 means files found.
   # 100 means invalid project this error should be caught earlier though.
   if [ $? -eq 0 ] ; then
      echo "No checked out files found."
   else
      sed -e 's{\\{/{g' "$SRC_OUT_FILE" | grep " Exc " > "$SRC_OUT_FILE.fix"
      while read FILE_NAME USER_ID PERMISSION DATE TIME FILE_PATH
      do
         # hopefully this will expand any truncated file names.
         FULL_FILE_NAME=`echo $FILE_PATH/$FILE_NAME*`
         # if the star was not expanded, use truncated form.
         if [ "$FULL_FILE_NAME" = "$FILE_PATH/$FILE_NAME*" ] ; then
            FULL_FILE_NAME="$FILE_PATH/$FILE_NAME"
         fi
         if [ "${FULL_FILE_NAME#* $FILE_PATH}" != "$FULL_FILE_NAME" ] ; then
            FULL_FILE_NAME="$FILE_PATH/$FILE_NAME"
         fi
         if [ "$SHOW_USER_NAME" = TRUE ] ; then
            echo -n "$USER_ID "
         fi
         if [ "$SHOW_DATE_TIME" = TRUE ] ; then
            echo -n "$DATE $TIME "
         fi
         if [ "$AS_PUT_COMMANDS" = TRUE ] ; then
            echo -n "put "
         fi
         if [ "$AS_UNGET_COMMANDS" = TRUE ] ; then
            echo -n "unget "
         fi
         echo "$FULL_FILE_NAME"
      done < "$SRC_OUT_FILE.fix"
   fi
   rm -f "$SRC_OUT_FILE" "$SRC_OUT_FILE.fix"
}

#===========================================================================
# Get the path to the directory above the current family.
#===========================================================================
function EZSrcGetCurrentFamilyParentDir
{
   CURRENT_ROOT=`EZGetCurrentFamilyRoot`
   CURRENT_ROOT="${CURRENT_ROOT%/*}" 
   if [ "$SUB_FAMILY_NAME_KLUDGE" != "" ] ; then   # this really sucks
      CURRENT_ROOT="${CURRENT_ROOT%/$SUB_FAMILY_NAME_KLUDGE}"
   fi
   echo "$CURRENT_ROOT"
}

#===========================================================================
# Cleanup any spurious files left hanging around.
# NOTE: A copy of this function exists in src_out.ksh and TestParsers.ksh
#===========================================================================
function PerformSpuriousFileCleanup
{
   FIRST_FILE=FALSE
   for SPURIOUS_FILE in `dlook.sh '*.exe.stackdump'`
   do
      if [ "$FIRST_FILE" = FALSE ] ; then
         echo
         echo "Successfully removing spurious files..."
         FIRST_FILE=TRUE
      fi
      rm -f "$SPURIOUS_FILE"
      echo "Successfully removed $SPURIOUS_FILE."
   done
}

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
SAVE_PWD=$PWD

SHOW_USER_NAME=FALSE
if [ "$FORCE_SHOW_USER" = TRUE -o \
     "$USER_LIST" = anyone -o \
     "${USER_LIST#*,}" != "$USER_LIST" ] ; then
   SHOW_USER_NAME=TRUE
fi

SPLIT_FAMILY_LIST=`echo $FAMILY_LIST | tr ',' ' '`
for ROOT_DIR in $EZ_FAM_ROOTS
do
   FAMILY_NAME="${ROOT_DIR##*/}"
   if [ "$FAMILY_LIST" = everywhere -a "$SUB_FAMILY_NAME_KLUDGE" != "" ] ; then
      if [ "${ROOT_DIR%$SUB_FAMILY_NAME_KLUDGE/*}" != "$ROOT_DIR" ] ; then
         continue
      fi
   fi
   EZIsInSpaceSeparatedList "$FAMILY_NAME" "$SPLIT_FAMILY_LIST"
   if [ $? -eq 0 -o "$FAMILY_LIST" = everywhere -o \
                    "$FAMILY_LIST" = universe ] ; then
      if [ ! -d $ROOT_DIR ] ; then
         echo "ERROR: Failed changing to directory $ROOT_DIR."
         continue
      fi
      if [ "$LIMIT_TO_CURRENT_DIR" != TRUE ] ; then
         cd $ROOT_DIR
      fi
      if [ "$FAMILY_LIST" = universe ] ; then
         FAMILY_NAME=universe
      fi
      PerformSpuriousFileCleanup
      EZSrcDisplayCheckedOutHeading "$FAMILY_NAME" "$USER_LIST"
      SPLIT_USER_LIST=`echo $USER_LIST | tr ',' ' '`

      if [ "$USER_LIST" != "$LOGNAME" ] ; then
         SRC_STATUS_ARGS="$SRC_STATUS_ARGS -show_user"
      fi
      for USER_NAME in $SPLIT_USER_LIST
      do
         if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
            src.ksh status $SRC_STATUS_ARGS -u "$USER_NAME"
         elif [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
            EZSrcListCheckedOutVSS "$FAMILY_NAME" "$USER_NAME" "$SHOW_USER_NAME"
         else
            echo -n "ERROR: Unsupported source control system "
            echo "$SOURCE_CONTROL_SYSTEM."
            EXIT_CODE=1
         fi
      done
      if [ "$FAMILY_LIST" = universe ] ; then
         break
      fi
   fi
done
cd "$SAVE_PWD"

EZSrcCleanup
exit $EXIT_CODE
