#===========================================================================
#  Filename: src_put.ksh
#    Author: Craig Schneider
# Orig Date: 08/04/04
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Put checked out files and/or directories into source control.
#===========================================================================

. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src put [-c] [-m Message] FileName(s)...
               [-dryrun] [-quiet | -verbose]

          -c: Create the file in source control, calls src create.
     -dryrun: Display the commands to be executed, without executing them.
 FileName(s): One or more file names to put into of source control.
          -m: The message to be used as the reason for checkin.
      -quiet: Suppress non critical messages about operations being performed.
    -verbose: Display verbose messages about operations being performed.

!
   exit 1
}

for ARGUMENT in "$@"
do
   # src put -c is an alias for src create (which ignores the -c). 
   if [ "$ARGUMENT" = "-c" ] ; then
      src.ksh create "$@"
      exit $?
   fi
done

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi
if [ $# -eq 0 ] ; then
   Usage
fi

integer SRC_FILE_LIST_COUNT=0
EXIT_CODE=0
MESSAGE_TEXT="None"

EZ_DRYRUN=FALSE
EZ_VERBOSITY=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -dryrun)
          EZ_DRYRUN=TRUE
          shift ;;
      -m) MESSAGE_TEXT="$2"
          shift
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) SRC_FILE_NAME_LIST[SRC_FILE_LIST_COUNT]="$1"
          SRC_FILE_LIST_COUNT=SRC_FILE_LIST_COUNT+1
          shift ;;
    esac
done

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
integer COUNT=0
while [ $COUNT -lt $SRC_FILE_LIST_COUNT ]
do
   SRC_FILE_NAME="${SRC_FILE_NAME_LIST[COUNT]}"
   COUNT=COUNT+1
   if [ ! -f "$SRC_FILE_NAME" -a ! -d "$SRC_FILE_NAME" ] ; then
      echo "ERROR: File $SRC_FILE_NAME was not found, checkin aborted."
      continue
   fi

   if [ ! -w "$SRC_FILE_NAME" ] ; then
      echo -n "File $SRC_FILE_NAME is not writable, checkin anyway <y,n>? "
      read ANSWER
      if [ "$ANSWER" != Y -a "$ANSWER" != y ] ; then
         echo "Skipping checkin of non-writable $(EZSrcObjectText $SRC_FILE_NAME)."
         continue
      fi
   fi

   EZSrcMappedFileNameSetup "$SRC_FILE_NAME" LOCAL
   if [ "$SOURCE_CONTROL_SYSTEM" = svn -o \
        "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
      EZInvokeCommand -out_err_file $SRC_OUTPUT $SRC_PUT_COMMAND \
              $SRC_MESSAGE_FLAG"$MESSAGE_TEXT"  "$SRC_MAPPED_FILE_NAME"
      if [ $? -eq 0 ] ; then
         if [ "$EZ_VERBOSITY" != quiet ] ; then
            echo "Successfully checked in $(EZSrcObjectText $SRC_FILE_NAME)."
         fi
      else
         cat $SRC_OUTPUT
         echo "ERROR: Failed checking in $(EZSrcObjectText $SRC_FILE_NAME)."
         EXIT_CODE=1
         egrep $EZ_EGREP_QUIET \
               "is not under version control|is not a working copy" $SRC_OUTPUT
         if [ $? -eq 0 ] ; then
            echo
            echo -n "   If your intent was to create the "
            echo    "$(EZSrcObjectText $SRC_FILE_NAME) in source control, "
            echo    "   please add the -c (Create) option on the command line."
            echo
         fi
      fi
      if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
         if [ ! -d "$SRC_FILE_NAME" ] ; then
            EZInvokeCommand -egrep_v \
                "' unlocked.|is not locked in this working copy|is not under version control" \
                $SRC_UNGET_COMMAND "$SRC_MAPPED_FILE_NAME"
         fi
      fi
   else
      echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
      EXIT_CODE=1
   fi
   EZSrcMappedDirNameRestore
done

EZSrcCleanup
exit $EXIT_CODE
