#===========================================================================
#  Filename: src_refresh.ksh
#    Author: Craig Schneider
# Orig Date: 09/13/04
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Refresh any files that have been updated in source control.
#      NOTE: This scripts needs to support being executed as well as dotted.
#            Pay attention to this when exiting or returning from functions.
#===========================================================================

if [ "$EZ_SOURCE_ENV" = "" ] ; then
   . $EZ_SHARED_ENV

   EZ_SOURCE_ENV="$(EZGetSrcScriptsDir)/settings_common.ksh"
   if [ ! -f "$EZ_SOURCE_ENV" ] ; then
      echo "ERROR: Failed to find source env file: $EZ_SOURCE_ENV."
      exit 1
   fi
fi
. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function EZSrcRefreshUsage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi

   cat <<!

Usage: src refresh [-a | -e | -f all | -f FamilyName(s)] [-dir | -dirtree]
               [-dryrun] [-quiet | -verbose]

          -a: Refresh all families, equivalent to '-f all'.  
        -dir: Refresh only the current directory.
    -dirtree: Refresh only the current directory and its sub directories.
     -dryrun: Display the commands to be executed, without executing them.
          -e: Refresh all families, equivalent to '-f all'.  
          -f: Comma separated list of family names or the word 'all'.
      -quiet: Suppress non critical messages about operations being performed.
    -verbose: Display verbose messages about operations being performed.

!
   if [ "$EZ_INTERACTIVE" != TRUE ] ; then
      exit 1
   else
      return 1
   fi
}

#===========================================================================
# List the files checked out by the specified user in the specified family.
#===========================================================================
function EZSrcRefreshClearHijacked
{
   return
}

#===========================================================================
# List the files checked out by the specified user in the specified family.
#===========================================================================
function EZSrcRefreshSVN
{
   FAMILY_NAME="$1"

   if [ ! -d ../.svn ] ; then
      echo
      echo "ERROR: Missing top level .svn directory for $PWD."
      echo
      return
   fi

   EZInvokeCommand -egrep_v "Updating '.'" \
        -sed_s "^At revision " "Successfully refreshed to revision " \
        $SRC_REFRESH_COMMAND 
}

#===========================================================================
# List the files checked out by the specified user in the specified family.
#===========================================================================
function EZSrcRefreshVSS
{
   FAMILY_NAME="$1"

   EZSrcRefreshClearHijacked

   FAM_SUB_DIR_PATH=
   RECURSIVE_OPTION="-R"
   if [ "$DIR_MODE" = TRUE ] ; then
      RECURSIVE_OPTION=
   fi
   if [ "$DIR_MODE" = TRUE -o "$DIRTREE_MODE" = TRUE ] ; then
      FAM_SUB_DIR_PATH="${PWD#$EZ_CURRENT_ROOT}"
   fi
   VSS_OUTPUT_FILE="$(EZPath2Win $TMPDIR/ss_refresh.$$)"
   EZInvokeCommand $SRC_REFRESH_COMMAND $RECURSIVE_OPTION \
      $EZ_RELEASE_ROOT/${FAMILY_NAME}${FAM_SUB_DIR_PATH} \
      "-O&$VSS_OUTPUT_FILE"
   cat "$VSS_OUTPUT_FILE" | egrep -v "as the default folder" | \
      sed -e "s{^Getting $EZ_RELEASE_ROOT/{Successfully extracted {" \
          -e "s{^Replacing local copy of $EZ_RELEASE_ROOT/{Successfully updated   {" \
          -e '/^\$\//d'

   rm -f "$VSS_OUTPUT_FILE"
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
function EZSrcRefreshMain
{
   if [ "$1" = -? ] ; then
      EZSrcRefreshUsage
      return 1
   fi

   EXIT_CODE=0

   EZ_DRYRUN=FALSE
   EZ_VERBOSITY=

   DIR_MODE=FALSE
   DIRTREE_MODE=FALSE
   FAMILY_LIST=`EZSrcGetCurrentFamilyName`

   typeset -l LOWERCASE_ARG
   while [ $# -ne 0 ]
   do
      LOWERCASE_ARG="$1"
      case $LOWERCASE_ARG in
         -dir)
             DIR_MODE=TRUE
             shift ;;
         -dirtree)
             DIRTREE_MODE=TRUE
             shift ;;
         -dryrun)
             EZ_DRYRUN=TRUE
             shift ;;
         -a|-e|-everywhere) FAMILY_LIST="everywhere"
             shift ;;
         -f) FAMILY_LIST="$2"
             if [ "$FAMILY_LIST" = all ] ; then
                FAMILY_LIST=everywhere
             fi
             shift
             shift ;;
         refresh)
             shift ;;
         -quiet|-verbose)
             EZ_VERBOSITY="${LOWERCASE_ARG#-}"
             shift ;;
         *)  if [ "$1" != "-?" ] ; then
                EZSrcRefreshUsage "Invalid argument: $1"
             else
                EZSrcRefreshUsage
             fi
             return 1 ;;
       esac
   done

   if [ "$FAMILY_LIST" = "" ] ; then
      EZSrcRefreshUsage \
      "ERROR: src refresh must be run with -f or executed under a valid family."
      return 1
   fi


   #===========================================================================
   # Perform the requested source control operations.
   #===========================================================================
   SAVE_PWD=$PWD

   SPLIT_FAMILY_LIST=`echo $FAMILY_LIST | tr ',' ' '`
   for ROOT_DIR in $EZ_FAM_ROOTS
   do
      FAMILY_NAME="${ROOT_DIR##*/}"

      EZIsInSpaceSeparatedList "$FAMILY_NAME" "$SPLIT_FAMILY_LIST"
      if [ $? -eq 0 -o "$FAMILY_LIST" = everywhere ] ; then
         cat <<!

Successfully refreshing family $FAMILY_NAME.
============================================================================
!

         if [ ! -d $ROOT_DIR ] ; then
            if [ "${ROOT_DIR#M:}" != "$ROOT_DIR" ] ; then
               echo -n "\nSuccessfully skipping refresh of "
               echo    "dynamic view $FAMILY_NAME.\n"
               continue
            fi
            if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
               TOP_DIR=`EZPath2Ksh "${ROOT_DIR%/*}"`
               \cd $TOP_DIR
               $SRC_REFRESH_COMMAND $FAMILY_NAME
            fi
            if [ ! -d $ROOT_DIR ] ; then
               mkdir -p $ROOT_DIR
            fi
         fi
         if [ "$DIR_MODE" != TRUE -a "$DIRTREE_MODE" != TRUE ] ; then
            ROOT_DIR_KSH=`EZPath2Ksh "$ROOT_DIR"`
            \cd $ROOT_DIR_KSH # need \cd since is dotted into the environment
         fi
         if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
            EZSrcRefreshSVN "$FAMILY_NAME"
         elif [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
            EZSrcRefreshVSS "$FAMILY_NAME"
         else
            echo -n "ERROR: Unsupported source control "
            echo    "system $SOURCE_CONTROL_SYSTEM."
            EXIT_CODE=1
         fi
      fi
   done
   \cd "$SAVE_PWD"  # need \cd since this is dotted into the environment

   EZSrcCleanup
   if [ "$EZ_INTERACTIVE" != TRUE ] ; then
      exit $EXIT_CODE
   else
      return $EXIT_CODE
   fi
}

if [ "$SRC_REFRESH_DOTTING" != TRUE ] ; then
   EZSrcRefreshMain "$@"   # not being dotted, execute directly.
   exit 0
fi
