#===========================================================================
#  Filename: src_rename.ksh
#    Author: Craig Schneider
# Orig Date: 09/27/04
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Change the name of a file or directory in source control.
#===========================================================================

. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src rename [-m Message] SourceFile TargetFile
                  [-dryrun] [-quiet | -verbose]

     -dryrun: Display the commands to be executed, without executing them.
          -m: The message to be used as the reason for source control changes.
  SourceFile: The file to be used to replace the contents of the TargetFile.
  TargetFile: The file to be updated from the contents of the SourceFile.
      -quiet: Suppress non critical messages about operations being performed.
    -verbose: Display verbose messages about operations being performed.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi
if [ $# -eq 0 ] ; then
   Usage
fi

EZ_DRYRUN=FALSE
EZ_VERBOSITY=
EXIT_CODE=0

MESSAGE_TEXT="Renamed by $LOGNAME using src rename on $(date '+%m/%d/%y')."
SOURCE_FILE_NAME=
TARGET_FILE_NAME=
SRC_OPTS=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -dryrun)
          EZ_DRYRUN=TRUE
          SRC_OPTS="$SRC_OPTS $1"
          shift ;;
      -m) MESSAGE_TEXT="$2"
          shift
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          SRC_OPTS="$SRC_OPTS $1"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) if [ "$SOURCE_FILE_NAME" = "" ] ; then
             SOURCE_FILE_NAME="$1"
          elif [ "$TARGET_FILE_NAME" = "" ] ; then
             TARGET_FILE_NAME="$1"
          else
             Usage "Too many file names specified."
          fi
          shift ;;
    esac
done

if [ "$SOURCE_FILE_NAME" = "" -o "$TARGET_FILE_NAME" = "" ] ; then
   Usage "Must specify both a SourceFile and TargetFile." 
fi

if [ ! -f "$SOURCE_FILE_NAME" -a ! -d "$SOURCE_FILE_NAME" ] ; then
   echo "ERROR: Failed to find SourceFile $SOURCE_FILE_NAME."
   exit 1
fi

if [ -f "$TARGET_FILE_NAME" -o -d "$TARGET_FILE_NAME" ] ; then
   echo "ERROR: TargetFile $TARGET_FILE_NAME already exists."
   exit 1
fi

SOURCE_FILE_DIRECTORY="${SOURCE_FILE_NAME%/*}"
if [ "$SOURCE_FILE_DIRECTORY" = "" -o \
     "$SOURCE_FILE_DIRECTORY" = "$SOURCE_FILE_NAME" ] ; then
   SOURCE_FILE_DIRECTORY="."
fi
if [ ! -d "$SOURCE_FILE_DIRECTORY" ] ; then
   echo "ERROR: SourceFile directory $SOURCE_FILE_DIRECTORY not found."
   exit 1
fi

TARGET_FILE_DIRECTORY="${TARGET_FILE_NAME%/*}"
if [ "$TARGET_FILE_DIRECTORY" = "" -o \
     "$TARGET_FILE_DIRECTORY" = "$TARGET_FILE_NAME" ] ; then
   TARGET_FILE_DIRECTORY="."
fi
if [ ! -d "$TARGET_FILE_DIRECTORY" ] ; then
   echo "ERROR: TargetFile directory $TARGET_FILE_DIRECTORY not found."
   exit 1
fi

EZSrcSetCheckoutActivity

#===========================================================================
# Check out the SourceFile dirctory if we need to.
#===========================================================================
SOURCE_CHECKED_OUT_DIR=FALSE

#===========================================================================
# Check out the TargetFile dirctory if we need to.
#===========================================================================
TARGET_CHECKED_OUT_DIR=FALSE

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
if [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
   EZSrcMappedFileNameSetup "$TARGET_FILE_NAME"
   SRC_MAPPED_TARGET_FILE="$SRC_MAPPED_FILE_NAME"
   EZSrcMappedDirNameRestore
   EZSrcMappedFileNameSetup "$SOURCE_FILE_NAME"
   SRC_MAPPED_SOURCE_FILE="$SRC_MAPPED_FILE_NAME"

   SOURCE_OBJ_TEXT="$(EZSrcObjectText $SOURCE_FILE_NAME)"
   EZInvokeCommand -out_err_file $SRC_OUTPUT \
      $SRC_RENAME_COMMAND $SRC_MAPPED_SOURCE_FILE $SRC_MAPPED_TARGET_FILE
   if [ $? -eq 0 ] ; then
      if [ "$EZ_VERBOSITY" != quiet ] ; then
         echo -n "Successfully renamed $SOURCE_OBJ_TEXT to "
         echo    "$(EZSrcObjectText $TARGET_FILE_NAME)."
      fi
   else
      cat $SRC_OUTPUT
      echo -n "ERROR: Failed renaming $(EZSrcObjectText $SOURCE_FILE_NAME) to "
      echo    "$(EZSrcObjectText $TARGET_FILE_NAME)."
      EXIT_CODE=1
   fi
   EZSrcMappedDirNameRestore
   
   if [ "$SOURCE_CHECKED_OUT_DIR" = TRUE ] ; then
      src put -m "renamed $(EZSrcObjectText $SOURCE_FILE_NAME)" $SRC_OPTS \
             "$SOURCE_FILE_DIRECTORY"
   fi

   if [ "$TARGET_CHECKED_OUT_DIR" = TRUE ] ; then
      src put -m "renamed $(EZSrcObjectText $TARGET_FILE_NAME)" $SRC_OPTS \
             "$TARGET_FILE_DIRECTORY"
   fi

   EZSrcCleanup
elif [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
   SOURCE_OBJ_TEXT="$(EZSrcObjectText $SOURCE_FILE_NAME)"
   EZInvokeCommand -out_err_file $SRC_OUTPUT \
      $SRC_RENAME_COMMAND "$SOURCE_FILE_NAME" "$TARGET_FILE_NAME"
   if [ $? -eq 0 ] ; then
      if [ "$EZ_VERBOSITY" != quiet ] ; then
         echo -n "Successfully renamed $SOURCE_OBJ_TEXT to "
         echo    "$(EZSrcObjectText $TARGET_FILE_NAME)."
      fi
   else
      cat $SRC_OUTPUT
      echo -n "ERROR: Failed renaming $(EZSrcObjectText $SOURCE_FILE_NAME) to "
      echo    "$(EZSrcObjectText $TARGET_FILE_NAME)."
      EXIT_CODE=1
   fi
   #EZInvokeCommand chmod 777 "$TARGET_FILE_NAME"
   src.ksh put -m "renamed $(EZSrcObjectText $SOURCE_FILE_NAME)" $SRC_OPTS .
   #src.ksh put -m "renamed $(EZSrcObjectText $TARGET_FILE_NAME)" $SRC_OPTS \
   #            "$TARGET_FILE_NAME"
else
   echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
   EXIT_CODE=1
fi

exit $EXIT_CODE
