#===========================================================================
#  Filename: src_repo_backup.ksh
#    Author: Craig Schneider
# Orig Date: 04/05/12
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Backup the source control repository.
#===========================================================================

. $EZ_SOURCE_ENV

BACKUP_DIRECTORY="$EZ_SUPER_DRIVE/SvnBackups"
BACKUP_MODE="RANGES"
REVISIONS_PER_RANGE=1000
NEW_RANGE=FALSE
START_REVISION=0
SEVEN_ZIP=TRUE
ONE_STEP=FALSE

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src repo_backup [-complete | -ranges] [-d BackupDirectory] [-dryrun] 
       [-new_range] [-range_size RevisionsPerRange] [-start_rev StartRevision]
       [-quiet | -verbose]

         -7z: Compress the dump file to 7z.
   -complete: Sets the backup mode to COMPLETE (default $BACKUP_MODE).
          -d: Backup directory (default $BACKUP_DIRECTORY).
     -dryrun: Display the commands to be executed, without executing them.
  -new_range: Specifies starting a new set of range files.
       -no7z: Do not compress the dump file to 7z.
      -quiet: Suppress non critical messages about operations being performed.
     -ranges: Sets the backup mode to RANGES (default $BACKUP_MODE).
 -range_size: Specify the number of revisions per range (default $REVISIONS_PER_RANGE).
  -start_rev: Specify the start revision to begin the dump(s) (default $START_REVISION).
    -verbose: Display verbose messages about operations being performed.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi

EXIT_CODE=0
EZ_DRYRUN=FALSE
EZ_VERBOSITY=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -7z)
          SEVEN_ZIP=TRUE
          shift ;;
      -complete)
          BACKUP_MODE="COMPLETE"
          shift ;;
      -dryrun)
          EZ_DRYRUN=TRUE
          shift ;;
      -d) BACKUP_DIRECTORY="$2"
          shift
          shift ;;
      -mode)
          BACKUP_MODE="$2"
          shift
          shift ;;
      -new_range)
          NEW_RANGE="TRUE"
          shift ;;
      -no7z)
          SEVEN_ZIP=FALSE
          shift ;;
      -one)
          ONE_STEP=TRUE
          shift ;;
      -ranges)
          BACKUP_MODE="RANGES"
          shift ;;
      -range_size)
          REVISIONS_PER_RANGE="$2"
          shift
          shift ;;
      -start_rev)
          START_REVISION="$2"
          shift
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          shift ;;
      *)  if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
    esac
done

if [ "$BACKUP_MODE" != RANGES -a "$BACKUP_MODE" != COMPLETE ] ; then
   Usage "Unsupported Backup Mode: $BACKUP_MODE"
fi
if [ ! -d "$BACKUP_DIRECTORY" ] ; then
   echo "ERROR: Backup directory $BACKUP_DIRECTORY does not exist."
   exit 1
fi

#===========================================================================
# Get the dump file base name.
#===========================================================================
function GetDumpFileBaseName
{
   typeset DUMP_TYPE="$1"
   typeset START_REVISION="$2"
   typeset END_REVISION="$3"

   typeset DUMP_FILE_BASE_NAME
   typeset -RZ6 START_REV_FMT
   typeset -RZ6 END_REV_FMT
   START_REV_FMT="$START_REVISION"
   END_REV_FMT="$END_REVISION"
   DUMP_FILE_BASE_NAME="SvnBackup${DUMP_TYPE}_${START_REV_FMT}_${END_REV_FMT}.dump"
   if [ "$DUMP_TYPE" = "Concat" ] ; then
      DUMP_FILE_BASE_NAME="SvnBackupFull.dump"
   fi

   echo "$DUMP_FILE_BASE_NAME"
}

#===========================================================================
# Get the dump file full name.
#===========================================================================
function GetDumpFileFullName
{
   typeset DUMP_TYPE="$1"
   typeset START_REVISION="$2"
   typeset END_REVISION="$3"

   typeset DUMP_FILE_BASE_NAME
   typeset DUMP_FILE_FULL_NAME
   DUMP_FILE_BASE_NAME=`GetDumpFileBaseName $DUMP_TYPE $START_REVISION \
                                                       $END_REVISION`
   DUMP_FILE_FULL_NAME="$BACKUP_DIRECTORY/$DUMP_FILE_BASE_NAME"

   if [ "$SEVEN_ZIP" = TRUE -a "$DUMP_TYPE" != "Concat" ] ; then
      DUMP_FILE_FULL_NAME="$DUMP_FILE_FULL_NAME.7z"
   fi
   echo "$DUMP_FILE_FULL_NAME"
}

#===========================================================================
# Perform the actual repository dump.
#===========================================================================
function PerformRepositoryDump
{
   typeset DUMP_TYPE="$1"
   typeset START_REVISION="$2"
   typeset END_REVISION="$3"

   DUMP_FILE_FULL_NAME=`GetDumpFileFullName $DUMP_TYPE $START_REVISION \
                                                       $END_REVISION`
   DUMP_OPTIONS="$EZ_SOURCE_CONTROL_ROOT -r $START_REVISION:$END_REVISION"
   if [ "$DUMP_TYPE" = "Incr" -o "$DUMP_TYPE" = "Concat" ] ; then
      DUMP_OPTIONS="$DUMP_OPTIONS --incremental"
   fi
   if [ "$EZ_VERBOSITY" != verbose ] ; then
      DUMP_OPTIONS="$DUMP_OPTIONS -q"
   fi
   if [ "$SEVEN_ZIP" = TRUE -a "$DUMP_TYPE" != "Concat" ] ; then
      DUMP_FILE_BASE_NAME=`GetDumpFileBaseName $DUMP_TYPE $START_REVISION \
                                                          $END_REVISION`
      DUMP_OPTIONS="$DUMP_OPTIONS | 7z a $DUMP_FILE_FULL_NAME -si$DUMP_FILE_BASE_NAME -bd | egrep -v 'Everything is Ok|Creating archive|Igor Pavlov|Compressing' | sed -e '/^$/d'"
   else
      if [ "$DUMP_TYPE" = "Concat" ] ; then
         DUMP_OPTIONS="$DUMP_OPTIONS >> $DUMP_FILE_FULL_NAME"
      else
         DUMP_OPTIONS="$DUMP_OPTIONS > $DUMP_FILE_FULL_NAME"
      fi
   fi

   if [ "$DUMP_TYPE" = "Concat" ] ; then
      if [ "$START_REVISION" = 0 -a -f "$DUMP_FILE_FULL_NAME" ] ; then
         echo -n "ERROR: Revision 0 and file $DUMP_FILE_FULL_NAME "
         echo    "already exists."
         exit 1
      fi
      if [ "$START_REVISION" != 0 -a ! -f "$DUMP_FILE_FULL_NAME" ] ; then
         echo -n "ERROR: Revision is not 0 and file $DUMP_FILE_FULL_NAME "
         echo    "does not exist."
         exit 1
      fi
   else
      if [ -f "$DUMP_FILE_FULL_NAME" ] ; then
         if [ "$DUMP_TYPE" = "Incr" ] ; then
            echo "Successfully detected $DUMP_FILE_FULL_NAME exists."
            return
         else
            echo "ERROR: File $DUMP_FILE_FULL_NAME already exists."
            exit 1
         fi
      fi
   fi

   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "svnrdump dump $DUMP_OPTIONS"
   else
      eval svnrdump dump $DUMP_OPTIONS
      if [ $? -ne 0 ] ; then
         echo "ERROR: Failed creating $DUMP_FILE_FULL_NAME."
         exit 1
      fi
   fi
   if [ "$DUMP_TYPE" = "Concat" -a "$START_REVISION" != 0 ] ; then
      echo -n "Successfully appended revisions $START_REVISION-"
      echo    "$END_REVISION to $DUMP_FILE_FULL_NAME."
   else
      echo "Successfully created  $DUMP_FILE_FULL_NAME."
   fi
}

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
   SVNRDUMP_EXE=`whence svnrdump 2>$EZ_DEV_NULL`
   if [ "$SVNRDUMP_EXE" = "" ] ; then
      echo "ERROR: Failed to find svnrdump executable."
      exit 1
   fi

   REPOSITORY_REVISION=`EZSrcGetRepositoryRevision`
   if [ "$REPOSITORY_REVISION" = "" ] ; then
      echo "ERROR: Failed to determine repository revision."
      exit 1
   fi

   echo "Successfully detected the repository revision is $REPOSITORY_REVISION."
   if [ "$BACKUP_MODE" = COMPLETE ] ; then
      PerformRepositoryDump Full $START_REVISION $REPOSITORY_REVISION
   elif [ "$BACKUP_MODE" = RANGES ] ; then
      LAST_REV_FILE_FULL_NAME="$BACKUP_DIRECTORY/SvnBackupFull.rev"
      if [ ! -f "$LAST_REV_FILE_FULL_NAME" ] ; then
         if [ "$NEW_RANGE" = TRUE ] ; then
            let INITIAL_REV=START_REVISION-1
            echo "$INITIAL_REV" > "$LAST_REV_FILE_FULL_NAME"
         else
            echo -n "ERROR: Failed to find Last Revision file "
            echo    "$LAST_REV_FILE_FULL_NAME."
            echo    "ERROR: Use -new_range if this is the initial backup."
            exit 1
         fi
      fi
      LAST_REVISION=`cat "$LAST_REV_FILE_FULL_NAME"`
      echo -n "Successfully detected the last revision backed up "
      echo    "was $LAST_REVISION."
      
      let END_REVISION=START_REVISION+REVISIONS_PER_RANGE
      if [ $START_REVISION -eq 0 ] ; then
         let END_REVISION=REVISIONS_PER_RANGE-1
      fi

      PREV_START_REVISION=$START_REVISION
      while [ 1 -eq 1 ]
      do
         if [ $END_REVISION -gt $REPOSITORY_REVISION ] ; then
            let END_REVISION=REPOSITORY_REVISION
         fi
         PerformRepositoryDump Incr $START_REVISION $END_REVISION
        
         PREV_START_REVISION=$START_REVISION
         let START_REVISION=END_REVISION+1
         let END_REVISION=END_REVISION+REVISIONS_PER_RANGE
         if [ $START_REVISION -gt $REPOSITORY_REVISION ] ; then
            break
         fi
         if [ "$ONE_STEP" = TRUE ] ; then
            break
         fi
      done

      if [ $LAST_REVISION -lt $REPOSITORY_REVISION ] ; then
         PREV_INCR_DUMP_FILE_FULL_NAME=`GetDumpFileFullName "Incr" \
                                        $PREV_START_REVISION $LAST_REVISION`
         if [ ! -f "$PREV_INCR_DUMP_FILE_FULL_NAME" ] ; then
            let PREV_PREV_START_REVISION=PREV_START_REVISION-REVISIONS_PER_RANGE
            PREV_INCR_DUMP_FILE_FULL_NAME=`GetDumpFileFullName "Incr" \
                                   $PREV_PREV_START_REVISION $LAST_REVISION`
         fi
         if [ -f "$PREV_INCR_DUMP_FILE_FULL_NAME" ] ; then
            rm -f "$PREV_INCR_DUMP_FILE_FULL_NAME"
            echo "Successfully deleted  $PREV_INCR_DUMP_FILE_FULL_NAME."
         fi

         let NEXT_REVISION=LAST_REVISION+1 
         PerformRepositoryDump Concat $NEXT_REVISION $REPOSITORY_REVISION
         if [ "$EZ_DRYRUN" != TRUE ] ; then
            echo "$REPOSITORY_REVISION" > "$LAST_REV_FILE_FULL_NAME"
            echo "Successfully set last revision to $REPOSITORY_REVISION."
         fi
      else
         echo "Successfully detected no repository changes to backup."
      fi
   else
      echo "ERROR: Unsupported Backup Mode $BACKUP_MODE."
      exit 1
   fi
else
   echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
   EXIT_CODE=1
fi

exit $EXIT_CODE
